import numpy as np
import matplotlib.pylab as plt
from inspect import isfunction

#if priorfunc == -1 then just use random walk prior,
#else priorfunc should be an array of the prior with one element for each frequency
def gp(p,sd,freq,p0=1.0,dw=1.0,w0=1.0,plot_tit='G_plot.pdf',xline=[],labann=[],
priorfunc = -1,square=1):


 cfur = p[0:-1:2]
 sd_cfur = sd[0:-1:2]
 sfur = p[1::2]
 sd_sfur = sd[1::2]
 fnow = freq
 
 
 
 if (type(priorfunc) == int):
  sig0_2 = 0.5*p0*dw*((w0/2/np.pi)/fnow)**2
 else:
  sig0_2 = 1.*priorfunc
 
 sig_c2 = sd_cfur**2
 sig_s2 = sd_sfur**2
 
 
 Gsk = ( sig0_2 / ( sig0_2 + sig_s2 ) - 0.5)*2
 Gck = ( sig0_2 / (sig0_2 + sig_c2) - 0.5)*2
 
 if (square == 0):
  Gsk = np.sqrt(Gsk.clip(min=0))
  Gck = np.sqrt(Gck.clip(min=0))
  
  #Gsk = np.sqrt(( sig0_2 / ( sig0_2 + sig_s2 ) - 0.5)*2)
  #Gck = np.sqrt(( sig0_2 / (sig0_2 + sig_c2) - 0.5)*2)
  #Gsk = ( np.sqrt(sig0_2) / ( np.sqrt(sig0_2) + np.sqrt(sig_s2) ) - 0.5)*2
  #Gck = ( np.sqrt(sig0_2) / (np.sqrt(sig0_2) + np.sqrt(sig_c2)) - 0.5)*2
 
 nfurtot = np.shape(Gck)[0]
 fig = plt.figure()
 ax2 = fig.add_subplot(111)
 ax2.plot(fnow,Gsk,label='Sk',color='r')
 ax2.plot(fnow,Gck,label='Ck',color='b')
 sum_gs = np.sum(Gsk)
 sum_gc = np.sum(Gck)
 sum_ave = (sum_gc + sum_gs)/2
 ax2.set_xlabel('frequency cycles/day')
 ax2.set_ylabel(r'$G= \frac{\sigma_0^2 - \sigma^2} {  \sigma_0^2 + \sigma^2 } $')
 ax2.set_title('G factor')
 ax2.text(0.98,0.55,r'$N_{\mathrm{eff}}=\sum_k \frac{G_{Sk} + G_{Ck}}{2} = '+np.str(np.int(sum_ave))+'$ of '+np.str(np.int(nfurtot)),ha='right',transform=ax2.transAxes,fontsize=14)
 ax2.set_xscale('log')
 ax2.set_yscale('log')
 ylim = list(ax2.get_ylim())
 for ian in range(len(xline)):
  ax2.plot([xline[ian]]*2,ylim,ls='--',color='k',label=labann[ian])
 
 plt.legend()
 plt.tight_layout()
 if (plot_tit == ''):
  plt.show()
 else:
  plt.savefig(plot_tit)
 
 return(sum_ave,Gsk,Gck)


import os
import numpy as np
# this code takes an input light curve
# truncates it into new light curves of length specified by the user.
# resaves the files

# hardwired options
equaldat = 1 #if equaldat = 1, the code will try to pick roughly even chunks but force them all to have the same number of data points

dir_default = os.getcwd()
dir = raw_input('Enter directory of data: ')
if not dir:
    dir = dir_default


os.chdir(dir)
fname = raw_input('Enter file name: ')


print 'light cruve will be chopped at multiples of the truncation time...'
trunctime = raw_input('Enter truncation time: ')
trunctime = np.double(trunctime)


dat = np.loadtxt(fname)
x   = dat[:,0]
y   = dat[:,1]
sig = dat[:,2]
ndat = x.shape[0]


xlen = x[-1] - x[0]

nchops = np.int( np.floor(xlen/trunctime) )

if (equaldat == 1):
    nequal = ndat/nchops

idxlo = 0
os.mkdir('chopped_'+fname[:fname.find('.')])
os.chdir('chopped_'+fname[:fname.find('.')])
for i in range(nchops):
    os.mkdir(str(i)+'_of_'+str(nchops))
    os.chdir(str(i)+'_of_'+str(nchops))
    
    if (equaldat ==1):
        idx = list(np.arange(idxlo,idxlo+nequal,1))
        idxlo = idxlo + nequal
        if (idxlo + nequal > ndat):
            break
    
    else: 
    	xchoplo  = i*trunctime
    	xchophi  = (i+1)*trunctime
    	idx      = np.where( (x >= xchoplo) and (x < xchophi) )[0]
    
    nidx     = len(idx)
    
    xnew   		= x[idx]
    ynew   		= y[idx]
    signew 		= sig[idx]
    datnew 		= np.zeros((nidx,3))
    datnew[:,0] = xnew
    datnew[:,1] = ynew
    datnew[:,2] = signew
    
    fnamenew = str(i)+'_of_'+str(nchops)+'_'+fname
    
    np.savetxt(fnamenew, datnew)
    os.chdir('../')

os.chdir(dir_default)
    
    



import numpy as np
import matplotlib.pylab as plt
import myvarlc as mv

file = '/Users/ds207/Google Drive/cont_222_thlag/katelc_group_1febmerge_8/rm369_gimerge/rm369_g_bok_G10_ccd4.dat'
#load light curve
dat = np.loadtxt(file)




#run script
time = dat[:,0]
flux = dat[:,1]
err  = dat[:,2]


a = mv.linvarfit(time,flux,err,diag=0)
print 'linvarfitop',a




#fig = plt.figure()
#ax1 = fig.add_subplot(111)
#ax1.errorbar(time,flux,err,ls='')
#plt.show()



#fit a power law to multiple target observations
#such that we fit y = y0_{1...N} x/x0^slope 
#i.e one slope but N normalisation offsets
#useful for multi object accretion disk lag satudies


import numpy as np



#inputs
#x[Ntarg,Ndim]
#y[Ntarg,Ndim]
#sigy[Ntag,Ndum]


#outputs
#os[Ntarg],sigos[Ntarg] the normalisations and variance
#slope, sigslope the slope and variance

def mypl_multi(x,y,sig,nits = 100):

 #now do analysis
 ntarg,ndim = np.shape(x)
   
 xlog    = np.log10(x)
 ylog    = np.log10(y)
 sigylog = sig/lag/np.log(10)
 wavl0 = np.sum(xlog/sigylog**2)/np.sum(1./sigylog**2)
 
 sy2 = sigylog**2
 
 nits = 100
 #use iterated optimal scaling to fit multi offset line
 slope = 1.0#4./3
 os = np.ones(ntarg)
 varos = np.ones(ntarg)
 
 xlwl0 = xlog-wavl0
 for it in range(nits):
  
  #update os
  for ip in range(ntarg):
   bot = np.sum(1./sy2[ip,:])
   osnew = np.sum((ylog[ip,:] - slope*xlwl0[ip,:])/sy2[ip,:])/bot
   varsnew = 1./bot
   os[ip] = osnew
   varos[ip] = varsnew
 
  ost = np.tile(os,(ndim,1)).T
  #update slope
  top = np.sum((ylog-ost)*xlwl0/sy2)
  bot = np.sum(xlwl0**2/sy2)
  slope = top/bot
  varslope = 1./bot
 
 
 
 return(os,varos,slope,varslope) 
 
 
 



## script to work out the eddington luminosity for an AGN with an input Mdot

import numpy as np

pi=np.pi
G=6.67384e-11
msun=1.9891e30
mp=1.67262e-27
c=2.9979e8
thomson_e=6.6524e-29
year=24.0*3600*365
watt2erg=1.0e7

## input umdot: accretion rate (M0 /yr)
##       um   : BH mass (M0)
## output ledd (Eddington luminosity in ergs s_1)
##        eddrat(Eddington ratio l/ledd)


def edd(um,umdot,eta):
#    ledd=4*pi*G*um*msun*mp/thomson_e *watt2erg * c
    ledd2=1.26e31*um*watt2erg
    l=eta*umdot/year*msun*c**2 *watt2erg
    
    eddrat= l/ledd2 
    
    return(l,ledd2,eddrat)
    

## input (IN LOG10) ummdot:mass times accretion rate (M0**2 /yr) with error ummdotsig
##       um   : BH mass (M0)  with error umsig
## if umlog or umsiglog < 0 then code assumes input is in linear NOT log units
## if ummdotversion = 1, code assumes ummdotlog and ummdotsiglog are in log units,
##... if -1 then ummdotlog and ummdotsiglog in linear units
##... if +2 the ummdotlog is actually umdotlog and ummdotsiglog is umdotsiglog
##... if -2 then ummdotlog is actually umdot and umdotsig
## output ledd (Eddington luminosity in ergs s_1)
##        eddrat(Eddington ratio l/ledd)
##        uncertainty in eddington ratio

def edd2(umlog,umsiglog,ummdotlog,ummdotsiglog,eta,ummdotversion=1):
#    ledd=4*pi*G*um*msun*mp/thomson_e *watt2erg * c
    
     

    
    if (umlog > 0):
     um = 10**umlog*(1. + 0.5*np.log(10)**2 * umsiglog*umsiglog)
    else:
     um = np.abs(umlog)
      
    if (umsiglog > 0):
     umsig = um*np.log(10.)*umsiglog
    else:
     umsig = 1.*umsiglog
     
    
    if (ummdotversion == 1):
     ummdot = 10**ummdotlog*(1. + 0.5*np.log(10)**2 * ummdotsiglog*ummdotsiglog)
     ummdotsig = ummdot*np.log(10.)*ummdotsiglog
     umdot=ummdot/um
     sigumdot=umdot*np.sqrt( (ummdotsig/ummdot)**2 + (umsig/um)**2 )
     
    elif (ummdotversion == -1):
     ummdot = 1.*ummdotlog
     ummdotsig = 1.*ummdotsiglog
     umdot=ummdot/um
     sigumdot=umdot*np.sqrt( (ummdotsig/ummdot)**2 + (umsig/um)**2 )
    elif (ummdotversion == 2):
     umdot = 10**ummdotlog*(1. + 0.5*np.log(10)**2 * ummdotsiglog*ummdotsiglog)
     sigumdot = umdot*np.log(10.)*ummdotsiglog
    elif (ummdotversion == -2):
     umdot = ummdotlog
     sigumdot = ummdotsiglog


    
    
    ledd2=1.26e31*um*watt2erg
    l=eta*umdot/year*msun*c**2 *watt2erg
    
    sigl=l/umdot*sigumdot
    sigledd2=ledd2/um*umsig
    
    
    
    eddrat= l/ledd2 
    sigeddrat=eddrat* np.sqrt( (sigl/l)**2 + (sigledd2/ledd2)**2 )
    
    
    
    umdotlog = ummdotlog - umlog
    umdotsig_log = np.sqrt(umsiglog*umsiglog + ummdotsiglog*ummdotsiglog)
    return(l,ledd2,eddrat,sigeddrat,umdot,sigumdot,umdotlog,umdotsig_log)
    
    
      
    
# input Mass, efficiency 
# output Eddington ratio
def eddrate(um,eta):
    eddmdot = 1.26e31 * um /eta / (c*c) # in Watts
    
    eddumdot = eddmdot * year / msun
    
    return(eddumdot)


#routine to calculate the mdot for a given M and eddington ratio !!
#inp um (black hole mass in solar masses)
#er eddington ratio (0-1)
#eta black hole accretion efficiency typically 0.1
#op  umdot solar masses per year

def ermin_mdotout(um,er,eta=0.1):
 umdot = er/460.7e6 / eta * um
 return(umdot)



from sklearn import decomposition
import numpy as np





#generate n fake light curves with different levels of correlation to the 'flows'
r = [0.8,0.5,0.2,0,0,0,0,0]

import mylcgen as mlc
flows = mlc.mylcgen(datfile='',p0=1.0,f0=0.1,a=-2,b=-2,tlo=0,thi=1100,dt=30.0,ploton=0,iseed=-1,meannorm = -1., sdnorm = -1.0)
flowmean = np.mean(flows[:,1])
flowrms  = np.std(flows[:,1])


nflow = np.shape(flows[:,0])[0]
x_op = [flows[:,0]]

#the constituent signals will be a combination of gaussian noise and the original signal
#The proportion of the original signal represented by the time series is represented by
#r. r = 1 means the signal has no noise and entirely represents the original time series
#r = 0 means the signal is pure noise.
for rnow in r:
 noise = np.random.randn(nflow)*flowrms*np.sqrt(1.-rnow)
 x     = noise + (flows[:,1]-flowmean)*np.sqrt(rnow)/flowrms
 x_op.append(x)
x_op = np.array(x_op).T


#use vector auto regression model on same data
from statsmodels.tsa.api import VAR, DynamicVAR







model = VAR(x_op)




import numpy as np
from pylab import *

# script to make a skewed gaussian.

n=2000
lo=-10.0
hi=10.0
a=4


def phi(x):
    phi=1/np.sqrt(2*np.pi) *e**(-x*x/2)
    return(phi)
    


x=np.arange(lo,hi,(hi-lo)/n)

    
y=2*phi(x)*phi(a*x)


plot(x,y)

show()



def gaus(x,x0,w,b,dx):
    y=e**(-np.log(2*(np.log(1.+2.*b*(x-x0)/dx)*1/b)**2))
    return(y)
    
    
x0=1.0
w=0.5
b=1.0
dx=0.1

y=gaus(x,x0,w,b,dx)


# python implementation of iterated optimal scaling
# we input a light curve (t,x,sig), a frequency f, and get out a sin and cosine amplitude and uncertainties for each
import numpy as np

#t=np.arange(1,100,0.5)
#n=t.shape[0]
#x=np.sin(0.5*t)
#sig=np.ones(n)*0.2
#f=0.5/2/np.pi

# function to fit the best s_k sin(w_k t) + c_k cos(w_k t) curve it can
# input f (frequecny NOT angular), t,x,sig times,fluxes,erros (1d arrays), 
# output s,sigs,c,sigc output sin and cosine amplitudes with uncertainties (all doubles)

def myoptscal(t,x,sig,f):
    #the bit above this is just a test to see if the thing below works. It will then be turned into a function
    sig2=sig*sig
    maxnits=100
    s=x.mean()
    sigs=0.1*s
    c=x.mean()
    sigc=0.1*c
    
    
    w=2*np.pi*f
    sw=np.sin(w*t)
    sw2=sw*sw
    cw=np.cos(w*t)
    cw2=cw*cw
    convfrac = 0.01 # 1 percent variation = convergence
    
    sconv = np.array([])
    cconv = np.array([])
    for idx in range(maxnits):
    #    print idx, s,sigs,c,sigc
    # deal with sin term    
        top = np.sum( (x-cw*c)*sw/sig2 )  
        bot = np.sum( sw2/sig2 )
    #    print top,bot,idx 
        s    = top/bot
        sigs = 1/bot
    
    # this is what happens if 0 freq is included    
        if ( (top == 0) and (bot == 0) ):
            s=0.0
            sigs=0.0
    # end of 0 freq problem 
        
    # Now deal with cosine term    
        top = np.sum( (x-sw*s)*cw/sig2 )
        bot = np.sum( cw2/sig2 )
        
        c    = top/bot
        sigc = 1/bot
        
        
    # now implement convergence check ########
        sconv = np.append(sconv,s)
        cconv = np.append(cconv,c)
        if (idx > 0):
            smean = sconv.mean()
            cmean = cconv.mean()
            sfrac = (s - smean)/smean
            cfrac = (c - cmean)/cmean
            
            if ( (np.abs(sfrac) < convfrac) and (np.abs(cfrac) < convfrac) ):
                break
                
            elif (idx == maxnits):
                print 'myoptscal.py warning: optimal scaling has not converged after',maxnits,' iterations!'
    # convergence check complete  ############          
    #    print s,sigs,c,sigc
    #    print ''
    #    print ''
    # print s,sigs,c,sigc
    # return(s,sigs,c,sigc)
    return(s,sigs,c,sigc)
        
        
        
    
    
    
    
    
    
    



import json, os, sys, logging
import pandas as pd, numpy as np
import sqlalchemy
from tqdm import tqdm
from pandas.io.sql import SQLTable
import matplotlib.pylab as plt
import os
import matplotlib.gridspec as gridspec
import vortexa_ccf_simple as mccf
import vortexa_convert_df2lc as c2l
import my_kateccf_py3 as mc3
import vortexa_forecast_tools as vs
import datetime
import vortexa_shift_time as vst
import vortexa_combine_signal as vcs
import vortexa_improve_lag_idea as rwlag

#format should be a list of data frames where each frame
#has 2 or 3 axes (1st is the time as a date time object, 2nd are the flux values,
#3rd (optional) are the uncertainties . Good to have these for CCF otherwise a 
#certain amount of guess work will ensue.
#20/11/2018 added forecast period. If forecast_period = , do no forecasting
#...else perform a forecast of 'forecast_period' days ahead

def vortexa_lag_analysis(df_in,df_titles='',figure_title = 'vortexa_lags_summary.pdf',
dateref = -1,laglim=[-50,50],gridfig=[16,12],
figures=[['driving time series','','Flow (tonnes)'],
['response time series','Date','Arbitrage'],
['ccf simple','',''],
['mean lags','Arbitrage forecast month','lag (days)'],
['ccf frrss','lag (days)',''],
['power spectrum','P(f)','frequency (cycles/day)']
],
figure_positions_x=[[0,12],[0,12],[0,3],[4,8],[9,12],[0,3]],
figure_positions_y=[[0,3],[4,7],[9,12],[9,12],[9,12],[13,16]],
sigmode = 0.2,
lagstat='peak',
nsim = 5000,
histbins = 20,
global_titles = ['']*100,
forecast_period = 0.0,
datemin_force = '',
datemax_force = '',
normalise = False
):
 
 figheads = [figures[i][0] for i in range(len(figures))]
 try:
  id_drive = figheads[0].index('driving time series')
 except:
  id_drive = -1
  
 try:
  id_echo = figheads.index('response time series')
 except:
  id_echo = -1
 
 try:
  id_ccf = figheads.index('ccf simple')
 except:
  id_ccf = -1
 
 try:
  id_lag = figheads.index('mean lags') 
 except:
  id_lag = -1
  
 try: 
  id_ccf_frrss = figheads.index('ccf frrss') 
 except:
  id_ccf_frrss = -1

 try: 
  id_pspec = figheads.index('power spectrum') 
 except:
  id_pspec = -1
  

  
  
  
  
  
 #initialise lists to store the uncertainties from the forecasts
 forecast_times = []
 forecast_uncerts = []
 forecast_uncerts_incarb = []
   
   
 #make plots of OB vs time and flows vs time for each region
 ndown  = gridfig[0] 
 nalong = gridfig[1]
 
  
 #insert list of data frames where each data frame has a time,value, uncertainty axis
 #measure lags relative to first
 nframes = len(df_in)
 necho   = nframes - 1
 color=['k','r','b','cyan','purple','orange','gree','skyplue','magenta']*nframes

 #must have at least 2 data frames to perform lag analysis
 if (nframes < 2):
  print('Must have at least 2 data frames to perform lag analysis in vortexa_lag_figures.py')
  return()

  
 #convert dates to datetime format
 df = []#list(df_in)
 for i in range(nframes):
  time = pd.to_datetime(df_in[i].iloc[:,0])
  dat  = df_in[i].iloc[:,1]
  
  if (normalise == True):
   dat = (dat - np.mean(dat))/np.std(dat)
  
  df.append( pd.concat([time,dat],axis=1) )
  #df[i].iloc[:,0] = pd.to_datetime(df[i].iloc[:,0])
  if (( datemin_force != datetime.datetime) & (datemax_force !='')):
   df[i]= df[i][(df[i].iloc[:,0] > pd.Timestamp(datemin_force)) & (df[i].iloc[:,0] < pd.Timestamp(datemax_force))]
   print(min(df[i].iloc[:,0]),max(df[i].iloc[:,0]),'min max forced dates')
 #save the combine forecast array using time shift
 
 
 #print(df[0].iloc[:10,:])
 #print(df[1].iloc[:10,:])
 #input('test break here')
 #print(df_in[0].iloc[:10,:])
 #print(df_in[1].iloc[:10,:])
 #input('test break here') 

 combine_x = []
 combine_noise = []
 combine_r = [1.]
 mape_save = []
  
 #perform forecasting if required
 lc_forecast = [[] for i in range(nframes)]
 if (forecast_period > 0):
  idx = 0
  for dfnow in df_in:
   
   
   a_s = vs.sarima(dfnow,orderin=(1,1,0),seasonal_order = 'auto',trend='c',enforce_invertibility=False,
   alpha_confidence=0.32,time_forecast=forecast_period,diagnostic_plot='')
   
   
   as0 = a_s[0]
   
   a_s = vs.polyfit(dfnow,order='test',time_forecast=forecast_period,alpha_confidence=0.32,frac_check=0.9)
   print(dfnow.values[-1,0],'last time')
   print(as0,'sarima')
   print(a_s[0],'poly')
   #input()
   mape_save.append(a_s[4])
   a = a_s[:4]
   lc_forecast[idx] = a
   
   idx = idx + 1

   
 #identify earliest date from all the time axis and use this as the first time by default
 datemax_save = []
 if (dateref == -1):
  dates = pd.concat([df[i].iloc[:,0] for i in range(nframes)])
  datemax_save.append([df[i].iloc[-1,0] for i in range(nframes)])
  datemin = min(dates)
  datemax = max(dates) + datetime.timedelta(days=forecast_period)
  date_trunc = np.min(np.array(datemax_save))
  print ('Earliest dat in sample...',datemin)
 else:
  datemin = dateref[0]
  datemax = dateref[1] + datetime.timedelta(days=forecast_period)
  date_trunc = dateref[1]
 #truncate the figures to make both the arb and flows run to the same time
 

 


 
 #if no titles supplied then make up here
 if (df_titles == ''):
  dataframe_titles = [np.str('Dataframe '+np.str(i)) for i in range(nframes)]
 else:
  dataframe_titles = df_titles
 
 #initialise figures
 gs1 = gridspec.GridSpec(ndown,nalong)
 gs1.update(left=0.15,right=0.85,wspace=0.05,hspace=0.0,bottom=0.1,top=0.99)
  
 
 #deal with the driving time series
 df_drive_time = df[0].iloc[:,0]
 df_drive_x    = df[0].iloc[:,1]
 if (len(df_drive_time)==0):
  print('first list element (driving time series) of input data frame cannot be empty')
  return()
  
 #return(df)
 lc_drive = c2l.convert_df_2_lightcurve(pd.concat([df_drive_time,df_drive_x],axis=1),reftime=datemin,noise = 0.01,normalise = 1)
 lc_drive = np.array(lc_drive,dtype=float)
 
 
 if (id_drive > -1):
  fig_x = slice(figure_positions_x[id_drive][0],figure_positions_x[id_drive][1],1)
  fig_y = slice(figure_positions_y[id_drive][0],figure_positions_y[id_drive][1],1)
  ax1 = plt.subplot(gs1[fig_y,fig_x])
  #plot the error bars if present
  try: 
   df_sig_x = df[0].iloc[:,2]
   ax1.errorbar(df_drive_time,df_drive_x,df_sig_x,
   label=None,color=color[0])
  except:
   pass
  
  
  ax1.plot(np.array(df_drive_time),np.array(df_drive_x),label=None,color=color[0],marker='o',ls='',markersize=3)
  print('min max  drive info',np.min(df_drive_x),np.max(df_drive_x))
  #plot the forecast if present
  fc = lc_forecast[0]
  if (len(fc) > 0):
   time_out,ymean,ylo,yhi = fc
   ax1.plot(time_out,ymean,label=None,ls='--',color=color[0])
   ax1.fill_between(time_out,ylo,yhi,alpha=0.2,label='forecast (historic flows only)',color=color[0])
   #for it in range(np.shape(df_drive_time)[0]):
   # print(it,df_drive_time[it],df_drive_x[it])
   #for it in range(np.shape(time_out)[0]):
   # print(it,'fc',time_out[it],ymean[it])
   #input()
   for it in range(np.shape(time_out)[0]):
     print('fc drive',it,time_out[it],ylo[it],ymean[it],yhi[it])
   
   ##indicate start of forecast period
   #ax1_b = ax1.twiny()
   #xl1 = list(ax1.get_xlim())
   yl1 = list(ax1.get_ylim())
   #ax1_b.set_xlim(xl1)
   #ax1_b.set_ylim(yl1)
   #ax1_b.set_xticklabels(['forecast start'])
   #ax1_b.set_yticklabels([])
   #ax1_b.set_yticks([])
   #ax1_b.set_xticks([time_out[0]])
   #ax1_b.plot([time_out[0]]*2,yl1,label=None,color='k',ls='--')
   #ax1_b.xaxis.set_tick_params(labelbottom='on',labeltop='off')
   ax1.plot([time_out[0]]*2,yl1,label=None,color='k',ls='--') 
  
  
  ax1.set_xlabel(figures[id_drive][1])
  ax1.set_ylabel(figures[id_drive][2])
  ax1.legend(fontsize='x-small')
  #set the global title
  ax1.set_title(global_titles[0])
 
 
 
 
 
 
 
 
 
 #deal with each of the response time series'
 if (id_echo > -1):
  fig_x = slice(figure_positions_x[id_echo][0],figure_positions_x[id_echo][1],1)
  fig_y = slice(figure_positions_y[id_echo][0],figure_positions_y[id_echo][1],1)
  ax2 = plt.subplot(gs1[fig_y,fig_x])
  ax2.set_xlabel(figures[id_echo][1])
  ax2.set_ylabel(figures[id_echo][2])
  ax2.set_title(global_titles[id_echo])

 
 #initialise ccf example
 if (id_ccf > -1):
  fig_x = slice(figure_positions_x[id_ccf][0],figure_positions_x[id_ccf][1],1)
  fig_y = slice(figure_positions_y[id_ccf][0],figure_positions_y[id_ccf][1],1)
  ax3 = plt.subplot(gs1[fig_y,fig_x])
  #ax3.set_xlabel('lag (days)')
  #ax3.set_ylabel('cross correlation function') 
  ax3.set_xlabel(figures[id_ccf][1])
  ax3.set_ylabel(figures[id_ccf][2])

  ax3.set_title(global_titles[id_ccf])

 #initialise mean lag plot
 if (id_lag > -1):
  fig_x = slice(figure_positions_x[id_lag][0],figure_positions_x[id_lag][1],1)
  fig_y = slice(figure_positions_y[id_lag][0],figure_positions_y[id_lag][1],1)  
  ax4 = plt.subplot(gs1[fig_y,fig_x])
  ax4.set_xlabel('Arbitrage forecast month')
  ax4.set_ylabel('lag (days)')
  ax4.set_title(global_titles[id_lag])
  ax4.set_xlabel(figures[id_lag][1])
  ax4.set_ylabel(figures[id_lag][2])

 

 #initialise ccf frrrss histogram
 if (id_ccf_frrss > -1):
  fig_x = slice(figure_positions_x[id_ccf_frrss][0],figure_positions_x[id_ccf_frrss][1],1)
  fig_y = slice(figure_positions_y[id_ccf_frrss][0],figure_positions_y[id_ccf_frrss][1],1)
  ax_frrss = plt.subplot(gs1[fig_y,fig_x]) 
  ax_frrss.set_title(global_titles[id_ccf_frrss])
  ax_frrss.set_xlabel(figures[id_ccf_frrss][1])
  ax_frrss.set_ylabel(figures[id_ccf_frrss][2])

 
 
 #define lists to save the lag info
 tsave = []
 lagsave = []
 r_coefs = []
 
  
 lc_echo_save = []
 rwlag_save=[np.array([np.array(lc_drive[:,0]),np.array(df_drive_x),np.ones(np.shape(df_drive_x)[0])]).T]
 for i in range(necho):
  df_echo_time = df[i+1].iloc[:,0]
  df_echo_x = df[i+1].iloc[:,1]
  
  if (id_echo > -1):
   ax2.plot(np.array(df_echo_time),np.array(df_echo_x),color=color[i+1],label=None,ls='',marker='o',markersize=3) 
   try:#look for errorbars in echo light curves
    df_echo_sig = df[i+1].iloc[:,2]
    ax1.errorbar(df_echo_time,df_echo_x,df_echo_sig,
    label=None,color=color[i+1])
   except:
    pass
  
   #plot the forecast if present
   fc = lc_forecast[i+1]
   if (len(fc) > 0):
    time_out,ymean,ylo,yhi = fc
    ax2.plot(time_out,ymean,label=None,ls='--',color=color[i+1])
    ax2.fill_between(time_out,ylo,yhi,alpha=0.2,label=None,color=color[i+1])
    for it in range(np.shape(time_out)[0]):
     print('fc',i,it,time_out[it],ylo[it],ymean[it],yhi[it])
   
  #perform the ccf function and plot to the figure
  lc_echo = c2l.convert_df_2_lightcurve(df[i+1],reftime=datemin,noise = 0.01,normalise = 1)
  lc_echo = np.array(lc_echo,dtype=float)
  lc_echo_save.append(lc_echo)
  
  try:
   tccf,ccf,lo,mean,hi,idlo,idmax,idhi = mccf.ccf_simp(lc_drive,lc_echo,dt=1.0,r_crit = 0.8,laglim=laglim)
  except:
   print('problem with ccf opperation')
   tccf = np.array([0])
   ccf  = np.array([0])
   lo = 0
   mean = 0
   idlo = 0
   idhi = 0
   idmax = 0
   
   
  
  #attempt the frrss method (first try fake errorbars)
  if (id_ccf_frrss > -1):
   cadence_drive = np.mean(lc_drive[1:,0]-lc_drive[0:-1,0])
   cadence_std_drive = np.std(lc_drive[1:,0]-lc_drive[0:-1,0])
   cadence_echo  = np.mean(lc_echo[1:,0]-lc_echo[0:-1,0])
   cadence_std_echo = np.std(lc_echo[1:,0]-lc_echo[0:-1,0]),np.std(lc_echo[1:,0]-lc_echo[0:-1,0])
   print('frrss cadence drive (mean rms) ',cadence_drive, cadence_std_echo)
   print('frrss cadence response (mean rms) ',cadence_echo, cadence_std_echo)
   
   overlap_min = max(np.min(lc_drive[:,0]),np.min(lc_echo[:,0]))
   overlap_max = min(np.max(lc_drive[:,0]),np.max(lc_echo[:,0]))
   overlap_period = overlap_max-overlap_min
   
   lolim =  -overlap_period/4
   hilim =  overlap_period/4
   lag_range = [lolim,hilim]
   gridres = 0.5*(cadence_drive + cadence_echo)/2
   print('frrss overlap period (days) ',overlap_period,
   '    lower and upper lag limits ',lolim,hilim,
   '     grid resolution ',gridres)
   
   
   try:
    lag_grid,ccf_save,tlags_peak,rlagspeak,tlags_centroid = mccf.ccf_frrss(lc_drive,lc_echo,dt=gridres,fraction_rss=0.8,nsim = nsim)
   except:
    tlags_centroid = np.array([0])
    ccf_save = np.array([0])
    tlags_peak = np.array([0])
    rlagspeak = np.array([0])
    tlags_centroid = np.array([0])
    print('frrss ccf failed')
   
   
   if (lagstat == 'peak'):
    tlags_centroid = tlags_peak
    
   lagrange = hilim-lolim
   lag_margain = 0.1*lagrange
   idinc = np.where((tlags_centroid > lolim + lag_margain) & (tlags_centroid < hilim - lag_margain))[0]
   tlags_centroid = tlags_centroid[idinc]  
   r_coefs.append(np.array(rlagspeak[idinc]))
   print (np.shape(tlags_centroid),'cent shape')
   

   hist = ax_frrss.hist(tlags_centroid,bins=histbins,alpha=1.0,color=color[i+1],label=None,normed=False,histtype='step')
   histmax_ccf = hist[0].max()
   
   print(np.shape(lc_echo),np.shape(lc_drive),np.shape(tlags_centroid))
   if (np.shape(tlags_centroid)[0] == 0):
    print('no centroids calculated - porbably no overlap between time series')
    lo,med,hi = [0,0,0]
   else:
    lo,med,hi = np.percentile(tlags_centroid,[15.865,50,84.135])
   ax_frrss.plot([lolim+lag_margain]*2,[0,histmax_ccf],color=color[0],ls='--',lw=4,label=None)
   ax_frrss.plot([hilim-lag_margain]*2,[0,histmax_ccf],color=color[0],ls='--',lw=4,label=None)
   

   rwlag_save.append( np.array([np.array(lc_echo[:,0]),np.array(df_echo_x),np.ones(np.shape(df_echo_x)[0])]).T )
  
  
  
  
  
  
  
  
  
   
   
   #combine the driving time series and arbitrage weighted by the correlation coefficient
   custom_mean = [np.mean(df[0].iloc[:,1]),np.mean(df[i+1].iloc[:,1])]
   custom_rms =  [np.std(df[0].iloc[:,1]),np.std(df[i+1].iloc[:,1])]
   print('combination_mean...',custom_mean)
   print('combination_rms...',custom_rms)
   fc = lc_forecast[i+1]
   if (len(fc) > 0):
    time_out,ymean,ylo,yhi = lc_forecast[0]
    t0   = pd.to_datetime(time_out)
    y0   = ymean
    sig0 = (yhi - ylo)/2
    t1,y1,ylo1,yhi1 = fc
    sig1 = (yhi1 - ylo1)/2
    df1 = pd.DataFrame([t0,y0,sig0]).T
    df2 = pd.DataFrame([t1,y1,sig1]).T
    crap,x_combine,noise_combine = vst.shift_time(df1,df2,shift=0,renorm = 1,custom_mean = custom_mean,custom_rms = custom_rms)#return(lc2_op,x_combine,noise_combine)
    combine_r.append( np.mean(np.array(rlagspeak[idinc])) )
    if (i + 1 == 1):
     combine_x.append(x_combine[:,0])
     combine_noise.append(noise_combine[:,0])
     forecast_uncerts.append(noise_combine[:,0])
    combine_x.append(x_combine[:,1])
    combine_noise.append(noise_combine[:,1])
    
 
    if (i == necho - 1):
     
     #add the updated forecast to the driving light curve plot
     combine_x = np.array(combine_x).T
     combine_noise = np.array(combine_noise).T
     combine_r = np.array(combine_r)
     signal_out,signal_noise = vcs.comb_signal(combine_x,combine_noise,combine_r)#return(signal_out,signal_noise)
     forecast_uncerts_incarb.append(signal_noise)
     forecast_times = pd.to_datetime(t0)  

   
   if (id_ccf > -1):
    ax3.plot(tccf,ccf,color=color[i+1],label = dataframe_titles[i+1]+' ccf')
    ax3.plot([tccf[idlo],tccf[idhi]],[ccf[idlo],ccf[idhi]],label=None,color=color[i+1])
  
   #make summary plot of mean lags
   if (id_lag > -1):
    ax4.errorbar([i],[med],[[med- lo],[hi - med]],marker='o',label = dataframe_titles[i+1],color=color[i+1])
    tsave.append(i)
    lagsave.append(med)
  
  
  
  
  
  
  
  
  
  
  
 #apply the random walk lag detector
 #return(centroids_data,centroids_model,time_model,signal_model,error_model)
 
 try:
  rwlag_op = rwlag.lag_rwmod(rwlag_save,titles=[],
  monte_carlo_iterations=1,
  extra_frequencies = [],
  random_walk_gridres = 1.0,
  summary_plot = '',#figure_title[:-4]+'_rwlag.pdf',
  refdate=datemin)
  centroids_model,time_model,signal_model,error_model = rwlag_op[1],rwlag_op[2],rwlag_op[3],rwlag_op[4]
 except:
  centroids_model = [ np.array([0,0]) for it in range(nframes) ]
  time_model = [ np.array([datemin,datemax]) for it in range(nframes) ]
  signal_model = [ np.array([0,0]) for it in range(nframes) ]
  error_model = [ np.array([0,0]) for it in range(nframes) ]
  
  

 
 for i in range(nframes):
  tm_now = time_model[i]
  sm_now = signal_model[i]
  no_now = error_model[i]
  if (i == 0):
   ax1.plot(tm_now,sm_now,color=color[i],label=dataframe_titles[i]+' model')
   ax1.fill_between(tm_now,sm_now-no_now,sm_now+no_now,color=color[i],label=None,alpha=0.3)
  else:
   ax2.plot(tm_now,sm_now,color=color[i],label=dataframe_titles[i]+' model')
   ax2.fill_between(tm_now,sm_now-no_now,sm_now+no_now,color=color[i],label=None,alpha=0.3)
 for i in range(necho): 
  print('model info')
  print(tm_now)
  print(sm_now)
  print(no_now)
  try:
   ax_frrss.hist(centroids_model[i],bins=histbins,alpha=1.0,color=color[i+1],
   histtype='step',label=dataframe_titles[i],normed=False)
  except:
   print('problem computing frrss histogram')

    
  
  
  
 
 if (id_lag > -1):
  #ax4.plot(tsave,lagsave,color=color[0],label=None)
  ax4.plot(tsave,-30*np.array(tsave),color=color[0],ls='--',label='arbitrage expected lag')
  ax4.legend(fontsize='xx-small')
 
 
 if (id_ccf_frrss > -1):
  ax_frrss.legend(fontsize='xx-small')
  yl = list(ax_frrss.get_ylim())
  idx = 0
  ax_frrss.tick_params(labelleft=False)
  for lagnow in lagsave:
   #ax_frrss.plot([lagnow]*2,yl,color=color[idx+1],label=None)
   idx=idx+1
 
 #compute power spectrum
 tx = lc_drive[:,0]
 tmean = np.mean(tx[1:]-tx[:-1])
 x  = lc_drive[:,1]
 nx = np.shape(tx)[0]
 ft = np.fft.fft(x)
 pspec = np.real(np.conj(ft)*ft)
 freq = np.fft.fftfreq(nx,tmean)
 nf = np.shape(freq)[0]
 nf2 = np.int(nf/2)
 fhi = freq[nf2]
 flo = 1.e-10
 idinc = np.where((freq>flo) & (freq<fhi) & (pspec>1.e-10))[0]
 
 if (id_pspec > -1):
  fig_x = slice(figure_positions_x[id_pspec][0],figure_positions_x[id_pspec][1],1)
  fig_y = slice(figure_positions_y[id_pspec][0],figure_positions_y[id_pspec][1],1)
  axps = plt.subplot(gs1[fig_y,fig_x]) 
  #axps.set_xlabel('frequency (cycles/day)')
  #axps.set_ylabel('P(f) Power spectrum')
  axps.set_xlabel(figures[id_pspec][1])
  axps.set_ylabel(figures[id_pspec][2])

  
  
  axps.plot(freq[idinc],pspec[idinc],color=color[0],label=dataframe_titles[0])
  
  for i in range(necho):
   lcn = lc_echo_save[i]
   dtmean = np.mean(lcn[1:,0]-lcn[:-1,0])
   nechotemp = np.shape(lcn[:,0])[0]
   freq = np.fft.fftfreq(nechotemp,dtmean)
   idinc = np.where((freq>flo) & (freq<fhi))[0]
   nf = np.shape(freq)[0]
   nf2 = np.int(nf/2)
   fhi = freq[nf2]
   ft = np.fft.fft(lcn[:,1])
   pspec = np.real(np.conj(ft)*ft)[:nf2]

   
   axps.plot(freq[idinc],pspec[idinc],color=color[i+1],label=dataframe_titles[i+1])
  
  

  
  ylim = list(axps.get_ylim())
  xlim = list(axps.get_xlim())
  ps_sort = np.sort(pspec)
  
  ylim[0] =ps_sort[2]#max([10**-10,ylim[0]])


  axps.set_ylim(ylim)



  xrefs = [1./365,1./31.,1./7]
  xreflab =['1 year','1 month','1 week']
  #
  axps2 = axps.twiny()
  axps2.set_xlim(xlim)
  axps2.set_ylim(ylim)
  
  axps2.set_xticks(xrefs)
  axps2.set_xticklabels(xreflab)
  
  [axps2.plot([xn]*2,ylim,ls='--',color='k',lw=1,label=None,zorder=0) for xn in xrefs]
  axps.set_xscale('log')
  axps.set_yscale('log')
  #axps2.set_xscale('log')
  
  #try:
  # axps.set_xscale('log')
  # axps.set_yscale('log')
  # axps2.set_xscale('log')
  #except:
  # print('cannot log-scale power spectrum')
 
 if (id_drive > -1):
  if (forecast_period > 0):
   ax1.plot(t0,signal_out,color='purple',label=None)
   ax1.fill_between(t0,signal_out-signal_noise,signal_out+signal_noise,color='purple',label='forecast (inc arb model)',alpha=0.3)
  ax1.legend()
  ax1.set_xlim([datemin,datemax])
 if (id_echo > -1):
  if (forecast_period > 0):
   yl2 = list(ax2.get_ylim())
   ax2.plot([datemax - datetime.timedelta(days=forecast_period)]*2,yl2,label=None,color='k',ls='--')
  ax2.legend()
  ax2.tick_params(axis='x',rotation=30)
  ax2.set_xlim([datemin,datemax])
  if (forecast_period > 0):
   ax2_b = ax2.twiny()
   ax2_b.plot([datemax - datetime.timedelta(days=forecast_period)]*2,yl2,label=None,color='k',ls='--')
   ax2_b.set_xlim([datemin,datemax])
   ax2_b.set_ylim(yl2)
   ax2_b.set_xticklabels(['forecast start'])
   #ax2_b.set_yticklabels([])
   #ax2_b.set_yticks([])
   ax2_b.set_xticks([datemax - datetime.timedelta(days=forecast_period)])
   ax2_b.xaxis.set_tick_params(labelbottom='off',labeltop='on')
   ax2_b.set_xlim([datemin,datemax])
   ax2.set_xlim([datemin,datemax])
   ax1.set_xlim([datemin,datemax])
   ax1.plot([datemax - datetime.timedelta(days=forecast_period)]*2,yl1,label=None,color='k',ls='--')
   #ax1.plot([time_out[0]]*2,yl1,label=None,color='k',ls='--') 
  ax1.set_xticklabels([])
  ax1.set_xlabel('')
  #print(datemax,forecast_period)
  #print(ax2_b.get_xlim())
  #print(ax2.get_xlim())
  #print(ax1.get_xlim())
  #input()
 if (id_ccf > -1):
  pass
  #ax3.legend()
 
 if (id_lag > -1):
  #ax4.legend()
  pass
 
 
 
 
 
 plt.savefig(figure_title)
 
 #print(global_titles[0])
 #print( np.array(forecast_uncerts), 'mean',np.mean(forecast_uncerts) )
 #print( np.array(forecast_uncerts_incarb),'mean',np.mean(forecast_uncerts_incarb) )
 #input()
 ax1.set_xlim([datemin,datemax])
 ax2.set_xlim([datemin,datemax])
 if (forecast_period > 0):
  ax2_b.set_xlim([datemin,datemax])
 return(r_coefs,forecast_times,np.array(forecast_uncerts),np.array(forecast_uncerts_incarb),
 np.array(mape_save))
 #





 
 #optional output each light curve as a text file for further more specific analysis
 #...
 
#  
#import mylcgen as mlc
#flows = mlc.mylcgen(datfile='',p0=1.0,f0=0.1,a=-2,b=-2,tlo=0,thi=1100,dt=30.0,ploton=0,iseed=-1,meannorm = -1., sdnorm = -1.0)
#flowmean = np.mean(flows[:,1])
#flowrms  = np.std(flows[:,1])
#time = flows[:,0]
#
#tref = pd.Timestamp(year=2015,month=1,day=1)
#timeflows = [tref + pd.Timedelta(days=fl) for fl in time]
#
#timelag =42.0
#timeflows2 = [tref + pd.Timedelta(days=fl+timelag) for fl in time]
#
#
#
#meta_df = [ pd.concat([pd.DataFrame(timeflows),pd.DataFrame(flows[:,1])],axis=1) ]
#meta_df.append( pd.concat([pd.DataFrame(timeflows2),pd.DataFrame(flows[:,1])],axis=1)  )
#a_op = vortexa_lag_analysis(meta_df,figure_title = './figures/test_fake.pdf')
#
# 
  
   
 
 


#test with fake data

import numpy as np
import matplotlib.pylab as plt
import glob
import pandas as pd
import datetime
import statsmodels.api as sm
import mylcgen as mlc
import astropy.convolution as apc


def makefake(tlo,thi,dt,shift = [2,3],noise=0.1,iseed=-1):

 
 dat=mlc.mylcgen(datfile='',p0=1.0,f0=0.1,a=-2,b=-2,tlo=tlo,thi=thi,dt=dt,ploton=0,iseed=iseed,meannorm = 0, sdnorm = 1.0)
  
 nlag = len(shift)
 ntimes = np.int((thi - tlo)/dt + 1)
 output = []
 
 lg = np.arange(ntimes)*dt
 today = pd.Timestamp.today()
 for i in range(nlag):
  lagcent = shift[i]
  lagwide = dt
  
  
  datelo = today - pd.Timedelta(days=thi-tlo)
  dates = [datelo + pd.Timedelta(days=i) for i in range(ntimes)]
  times = np.arange(tlo,thi+dt,dt)
  shape_k = np.int(2*(ntimes-1) + 1)
  kernel = np.zeros(shape_k)
  k0 = np.int(np.floor(shape_k/2))
  laggrid = np.zeros(shape_k)
  print(k0,shape_k)
  laggrid[k0:]=lg
  laggrid[:k0]=lg[-1:0:-1]
  
  
  kernel= np.exp(-0.5*((laggrid-lagcent)/lagwide)**2)/(2*np.pi*lagwide)**2
  response = apc.convolve(dat[:,1],kernel)
  
  response_std = np.std(response)
  response_mean = np.mean(response)
  response = (response - response_mean)/response_std
  response = response + np.random.randn(ntimes)*noise
  
  
  if (i == 0):
   dsave_mean = np.mean(dat[:,1])
   dsave_std  = np.std(dat[:,1])
   dsave = (dat[:,1] - dsave_mean)/dsave_std
   dsave = dsave + np.random.randn(ntimes)*noise
   output.append(pd.DataFrame(list(np.array([dates,dsave]).T)))
   

  output.append(pd.DataFrame(list(np.array([dates,response]).T) )) 
  
  
 return(output)
 
 
#test
#a = makefake(0,100,1.0)






## module to return the black body plank function of an input temperature and wavelength
#output blackbody intensity (erg/cm2/s/Hz/ster)


#update 6/8/17 include feature to go from erg/cm^2/s/Hz to mJy
#update 6/8/17 include feature to go from erg/cm^2/s/Hz to erg/cm^2/s/A
#using conversions below http://www.stsci.edu/~strolger/docs/UNITS.txt
#[Y Jy]                        = 3.33564095E+04 * [X1 erg/cm^2/s/A] * [X2 A]^2
#[Y Jy]                       = 1.0E+26 * [X W/m^2/Hz]
#[Y Jy]                        = 1.0E+23 * [X erg/cm^2/s/Hz]
#[Y erg/cm^2/s/Hz]             = 3.33564095e-19 * [X2 A]^2 [X1 erg/cm^2/s/A]


import numpy as np
def bnu(wave,temp,mjy=1,ergcmang = 0):
 
	c1 = 1.43883e8  ### c1 = hc/k (in cgs units) *10^8 as the wavelength is given in angstroms
	c2 = 1.95722e5  ### c2 = (c1 / (2hc))**1/3
	
	
	##convert to si
	
	#c1=1.43916e-2
	#c2=1.9577e6
	BNU = 0.
	
	if (temp > 0.0):
		X = c1/(wave*temp)
		if X > 85:
			bnuln=3.*np.log((c1/wave)/c2)-X
			bnu=np.e**(bnuln)
		if X < 1e-4:
			factor= 2. / ( X * ( X + 2. ) )
			X = X *temp /c2
			bnu = factor * X**3
		if X < 85 and X > 1e-4:
			factor = 1./ (np.e**(X) - 1.)
			X = X *temp /c2
			bnu = factor * X**3
	else:
		bnu = 0.0
	
	
	if (mjy == 1):
		bnu = bnu * 1.e26# * wave*wave * 3.33564095e7
	elif (ergcmang ==1):
		bnu = bnu/3.33564095e-19 / wave/wave
	return(bnu)




#if you have an array of EITHER  wav or temp then use the vectorized version
def bnuvec(wav,temp,mjy=1, ergcmang = 0):
	a = np.vectorize(bnu)
	return( a(wav,temp,mjy=mjy, ergcmang = 0))




import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter
import myscatterhist
import mymedquart
import os

# hardwired parameters  !!!!!!!!!!!!!!!!!!


def myscatterhistbasic(dirmain,subfile,parmlab,idx1,idx2,chop,subfilelab):

 nbins = 100

 col = ['b','r','c','m','g','y','k','0.75','b','r','c','m','g']
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!

 os.chdir(dirmain)


 nplots = len(subfile)
 parm1 = []#np.zeros((0,nplots))
 parm2 = []#np.zeros((0,nplots))


# Load the data first to decide what plot limits to use
 for idx in range(nplots):
  
  dat = np.loadtxt(subfile[idx])
  parm1.append(dat[chop:,idx1])
  parm2.append(dat[chop:,idx2])
#
 
  print 'running scatterplot code',idx, len(parm1[0])
 #parm1 = np.random.randn(200000) + 10
 #parm2 = np.random.randn(200000)

# determine plot limits
 xloall=[]
 xhiall=[]
 yloall=[]
 yhiall=[]
 opx=[]
 opy=[]
 for idx in range(nplots):
  medloupx = mymedquart.mymedquart(parm1[idx],0.68)

  
  medloupy = mymedquart.mymedquart(parm2[idx],0.68)
  
  print medloupx
  print medloupy
  print 'med loup above are the medloup'
  raw_input()
  xlo = medloupx[1]
  xhi = medloupx[2]
  xmed = medloupx[0]
  xloall.append(xlo - np.abs(xlo)/10)
  xhiall.append(xhi + np.abs(xhi)/10)
  
  ylo = medloupy[1]
  yhi = medloupy[2]
  ymed = medloupy[0] 
  yloall.append(ylo - np.abs(ylo)/10)
  yhiall.append(yhi + np.abs(yhi)/10) 
  opx.append([xmed,xlo,xhi])
  opy.append([ymed,ylo,yhi])
 
 
 xlo = min(xloall)
 xhi = max(xhiall)
 ylo = min(yloall)
 yhi = max(yhiall)
 print xloall
 print xhiall
 print yloall
 print yhiall
 print 'above are the lowest pointsof all the data'
 
 
 raw_input()

 for idx in range(nplots):
  axHistx = myscatterhist.myscatterhist(parm1[idx],parm2[idx],col[idx],np.str(subfilelab[idx]),parmlab[idx1],parmlab[idx2],xlo,xhi,ylo,yhi,nbins)
 
  print 'finished', subfile[idx],subfilelab[idx]
#print plot
 axHistx.legend(prop={'size':10})
 
 


#os.chdir('../..')


 print 'limits', xlo,xhi,ylo,yhi
 plt.xticks(rotation = 90)

 plt.show()



#dirmain = './inclinations'
#subfile = ['op_tfbop_0.720','op_tfbop_0.730','op_tfbop_0.740','op_tfbop_0.750','op_tfbop_0.760','op_tfbop_0.770','op_tfbop_0.780']#,'11_of_12']
#parmlab = ['P0','w0','alpha','d_alpha','Error Bar Expansion Factor','Offset Level']





# python code to calculate the structure function (dy vs dx for pairs of points)
# Inps: x,y array
# Ops : dx, dy the sorted difference in x and y's (should be (nxy - 1)/2 values )
import numpy as np 

def mysf(x,y):

 nxy = x.shape[0]
 
 dx = np.array(())
 dy = np.array(())
 for i in range(nxy-1):
  #print i, x[i].shape[0]
  dxit = x[i+1:] - x[i]
  dyit = y[i+1:] - y[i]
  dx   = np.append(dx,dxit)
  dy   = np.append(dy,dyit)
  
  idx_sort = np.argsort(dx)
  dx   = dx[idx_sort]
  dy   = dy[idx_sort] 
 return(dx,dy)
 
 
 
 
### test it!
#
#import numpy as np
#import pylab as plt
#import mysf
#
#
#fname = 'testfile.dat'
#
#dat = np.loadtxt(dat)
#
#sf = mysf.mysf(x,y)
#dx = sf[0]
#dy = sf[1]
#
#fig = plt.figure()
#ax1 = fig.add_subplot(211)
#ax1.plot(x,y)
#
#ax2 = fig.add_subplot(221)
#ax2.plot(dx,dy)
#
#plt.show()
#
##



import numpy as np
import matplotlib.pylab as plt
import mytemp0 as mt0

twopi = np.pi*2
deg2rad = np.pi/180
planck = 6.626307004e-34
c      = 2.99792458e8
boltz  = 1.38064852e-23




def pytfb_sub(taus,embh,emdot,wavang, deginc, t0vin=-1, t0iin = -1, alpha_visc = -0.75, hxsch = 3.0, alpha_irad = -0.75, eta = 0.1, rlosch = 3.0, norm = 1):
 tauhi  = taus[-1]
 ntaus  = np.shape(taus)[0]
 psis   = np.zeros(ntaus)
 
 
#either input desired reference temperature t0 at 1 light day, or calculate the
#value appropriate for black body accretion disc given m and mdot 
 if (t0vin < 0):
  t0v   = mt0.tv0(embh,emdot)
 else:
  t0v   = t0vin
 
 if (t0iin < 0):
  t0i   = mt0.ti0(embh,emdot, eta = eta)
 else:
  t0i   = t0iin 
  

 #rlosch = 3.0
 #wavang = 4000.
 #hxsch  = 3.0
 
 
 nrad   = 10000
 naz    = 100
 #need to calculate 1/T^3 (r/r0)^alpha_irad x^2/wav^2 / (cosh(x) - 1) delta (tau - tau(r,theta)) dtau
 
 
 t0v2   = t0v*t0v
 t0v4   = t0v2*t0v2
 
 t0i2   = t0i*t0i
 t0i4   = t0i2*t0i2
 
 rsch   = 1.15821e-10*embh #scwarzchild radius in light days
 hx     = hxsch*rsch
 hx2    = hx*hx
 cosi   = np.cos(deginc*deg2rad)
 sini   = np.sin(deginc*deg2rad)
 hxci = hx*cosi
 hc_kwav = planck*c/wavang/1.e-10/boltz
 wavang2 = wavang*wavang
 rlold  = rlosch*rsch
 
 
 #print t0v,t0i
 
 #this estimate for the cutoff radius is based on the max radius of the
 #highest lag (re-arrange equation 3 in Starkey et al 2017)
 #should be ok but might exclude some significant low lags for a VERY hot, edge on disk
 rhisch = -1.* max((tauhi - hxci)/(1.+sini), tauhi) #if -ve then this is in light days
 #print rhisch
 if (rhisch < 0):
  rhild  = np.abs(rhisch)
 else:
  rhild  = rhisch*rsch
 
 
 nazsub1 = naz - 1
 
 #now define radius grid
 rgrid  = np.logspace(np.log10(rlold),np.log10(rhild),nrad)
 azgrid = np.linspace(0.0, twopi,naz)
 azgrid_s = azgrid[1:] - azgrid[:-1]
 #calculate temperature at each radius grid
 av4   = alpha_visc*4 
 ai4   = alpha_irad*4
 tv4   = t0v4*(rgrid)**av4
 ti4   = t0i4*(rgrid)**ai4
 rir0b = ti4/t0i4 #this is just (r/r0)^alpha_irad
 ttot4 = tv4 + ti4
 ttot2 = np.sqrt(ttot4) 
 ttot  = np.sqrt(ttot2)
 ttot3 = ttot2*ttot
 
 #each point will have a time delay and a weighting append these to a list
 #for each point in the disc
 
 #loop of azimuths
 delsave = []
 wsave   = []
 azwidth = twopi/nazsub1
 azsave=[]
 razsave=[]
 for i in range(nrad-1):
  
  #quick way 
  #tv4now    = tv4[i]
  #ti4now    = ti4[i]
  #ttotnow   = ttot[i]
  #ttot3now  = ttot3[i]
  
  
  radlo = rgrid[i]
  radhi = rgrid[i+1]
  drad  = radhi - radlo
  raz   = np.random.uniform(radlo,radhi,nazsub1)
  
  tv4   = t0v4*(raz)**av4 #modification for inner radius to the right (negligible)* (1 - np.sqrt(rlold/raz)) / (1 - np.sqrt(rlold/1))
  ti4   = t0i4*(raz)**ai4
  rir0b = ti4/t0i4 #this is just (r/r0)^alpha_irad
  ttot4 = tv4 + ti4
  ttot2 = np.sqrt(ttot4) 
  ttot  = np.sqrt(ttot2)
  ttot3 = ttot2*ttot
 
  
  daz  = np.sqrt(raz*raz + hx2)
  
  
  az   = np.random.uniform(nazsub1)*azgrid_s + azgrid[:-1]#np.random.uniform(azgrid[:-1],azgrid[1:],1)
  caz  = np.cos(az)
  tdl  = hxci - raz*caz*sini + daz
  

  
  x    = hc_kwav/ttot#hc_kwav/ttotnow
  x2   = x*x
  
  #the radlo * drad *azwidth is the solid angle element
  #azwidth can be left off here as it is always the same
  weight = rir0b/ttot3 * x2/wavang2/(np.cosh(x) - 1) * radlo*drad*azwidth      #rir0b[i]/ttot3now * x2/wavang2/(np.cosh(x) - 1)
  wsave.append(weight)
  delsave.append(tdl)
  
  azsave.append(az)
  razsave.append(raz)
  #introduce a lower threshold weight to abort the iterations (to save time)
  
 
 
 delsave = [item for sublist in delsave for item in sublist]
 wsave   = [item for sublist in wsave for item in sublist]
 azsave = [item for sublist in azsave for item in sublist]
 razsave   = [item for sublist in razsave for item in sublist]
 
 
 razsave = np.array(razsave)
 azsave = np.array(azsave)
 delsave = np.array(delsave)
 wsave   = np.array(wsave)
 nsave =np.shape(delsave)[0]
 dat = np.zeros((nsave,3))
 dat[:,0] = razsave
 dat[:,1] = azsave
 dat[:,2] = delsave
 np.savetxt('shittyprob.txt',dat)
 
 
 
 #save the results
 
 
 
 
 #add smoothing function to approximate delta function
 #ntaus = np.shape(taus)[0]
 sigtau = taus[1] - taus[0]
 sigtaulim = 2.5*sigtau
 
 
 for i in range(ntaus-1):
  taulo = taus[i]
  tauhi = taus[i+1]
  taulimlo = taulo - sigtaulim
  taulimhi = tauhi + sigtaulim
  taucent = (taulo + tauhi)/2
  idxsmooth = np.where((delsave >= taulimlo) & (delsave < taulimhi))[0]
  taunow    = delsave[idxsmooth]
  wnow      = wsave[idxsmooth]
  
  a = taunow - taucent
  b = sigtau
  aa_bb2 = a*a/b/b
  gauss     = np.exp(-aa_bb2/2)
  gsum      = np.sum(gauss)
 

  #WONT SOME SURFACE ELEMENTS BE COUNTED TWICE AND SO BE INCLUDED IN MORE THAN ONE GAUSSIAN WEIGHTING? YES BUT I DONT THINK THIS MATTERS
  psis[i] = np.sum( wnow*gauss/gsum )
  
  
  #print taus[i], psis[i], ntaus
 #if norm == 1then normalise so max == 1
 if (norm == 1):
  pt   = (psis - np.min(psis))
  psis = pt/np.max(pt)
 return(psis)







#outputs are psis[ntau] to input set of time grid[ntau]
#make code to test and plot 
 
import timeit 
wavang = 4000.
embh   = 1.e8
emdot  = 1.0
deginc = 0.0

t0v   = -1.6e4
t0i   = -1.2e4
taus   = np.arange(0.0,15.,0.05)



start = timeit.default_timer()
psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1)
stop = timeit.default_timer()



print 'run time', stop-start
 
fig = plt.figure()
ax1 = fig.add_subplot(111)


ax1.plot(taus,psis)
plt.savefig('pytfbplottest_2.pdf')



















#big fix nov 1st 2017 for very large black hole masses (schwarzchild radius bigger than 1 ld - reference radius and so 1 - rin/rld is negative. This screws up temperature radius law. Fix by setting temperature at all inner radii to zero and only using the scaling for radi above this

import numpy as np


#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!
##!!!!!!!!!!!!!!!!!!!!!!!!!!!!define T0 in T = T0 (r/r0)**alpha#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!
def temp0(dotmmlog, emlog, sigdotmmlog =0, sigemlog = 0.0, alpha_in = 0.75, sig_alphain=0.0,eta=0.1, alb=0, hxrs = 3,r0ld = 1):


 gnewt = 6.673e-11
 c     = 2.9979e8
 sig   = 5.67e-8
 emsun   = 2.e30 
 ld    = 2.59e13
 em    = 10**emlog
 sigem = em*np.log(10)*sigemlog
 
 
 rs    = 2*gnewt*em*emsun/c**2

 hx    = hxrs*rs
 
 if (r0ld < 0):
  r0    = np.abs(r0ld*rs)
 else: 
  r0    = r0ld*ld


 emsunyr = emsun/(3600.*24*365) 
 
 dotmm = 10**dotmmlog
 sigdotmm  = dotmm*np.log(10)*sigdotmmlog

 emdot = dotmm/em
 sigemdot = emdot*np.sqrt( (sigdotmm/dotmm)**2 + (sigem/em)**2 )
  
 
 a         = 3*gnewt*emsun*emsunyr/(8*np.pi*sig)
 aextra    = r0**(4*alpha_in)
 sigaextra = aextra*np.log(r0)*4*sig_alphain
 anew     = a/aextra
 siganew  = anew / aextra * sigaextra
 

 
 b        = anew*dotmm
 sigb     = b*np.sqrt( (siganew/anew)**2 + (sigdotmm/dotmm)**2)   
 
 #for i in range(np.shape(b)[0]):
 # print i,a,b[i],sigb[i]
 
 #print 'diagnostics'
 #print hx,(1.-a),eta,emsunyr,c**2,sig,r0
 #raw_input()
 
 c         = hx*(1.-alb)*eta*emsunyr*c**2/(4*np.pi*sig)
 cextra    = 1.*aextra
 sigcextra = 1.*sigaextra 
 cnew      = c/cextra
 sigcnew  = cnew /cextra * sigcextra
 

 d        = cnew*emdot
 sigd     = d * np.sqrt( (sigcnew/cnew)**2 + (sigemdot/emdot)**2 )

 #for i in range(np.shape(b)[0]):
 # print i,c,d[i],sigd[i]
 #raw_input() 
  
 baddc     = b+d 
 sig_baddc = np.sqrt(sigb*sigb + sigd*sigd)  
 
 temp0     = baddc**0.25
 sigtemp0  = temp0/baddc*0.25*sig_baddc
 #print sigtemp0,temp0, sig_baddc, baddc,'dsfdsfs', sigb,b, sigd,d, 'dfs', sigcnew,cnew, sigemdot,emdot
 
 
 return(temp0,sigtemp0,b**0.25,d**0.25)

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!#!!!!!!!!!!!!!!!!!!!!!!!!!!!!








#function to return the radius for a specific temperature r = (T1/T)^[1/alpha] r1
#can include errors on T1 and alpha
def rsub(Tsub, r1, T1, alpha, sigT1= 0.0, sigalpha = 0.0):

 alpha_1     = 1./alpha
 alpha_1_sig = alpha_1 * sigalpha / alpha 
 
 T1_T = T1/Tsub
 
 T1_T_alpha_1     = T1_T**alpha_1
 
 T1_T_alpha_1_sig = T1_T_alpha_1 * np.sqrt( (alpha_1/T1_T * sigT1/Tsub)**2 + (np.log(T1_T)*alpha_1_sig)**2 )
 
 
 a = T1_T_alpha_1 * r1
 siga = T1_T_alpha_1_sig * r1
 
 return(a,siga)




def tv0(em,emdot,r0=1.0,rinsch=3.0):
 sb      = 5.670367e-8
 gnewt   = 6.67408e-11
 msun    = 1.98855e30
 secyr   = 31557600.0
 ld      = 2.59020683712e+13
 c       = 299792458.0
 rinld   = rinsch * 2*gnewt*msun*em/c/c/ld
 
 if (rinld > r0):
  r0in = np.ceil(rinld/r0)*r0
  print('mytemp0.py inner radius bigger than reference radius (Mbh too big)  changing reference radius')
 else:
  r0in = r0
 
 
 
 tv0out4 = em*emdot*3*gnewt*msun*msun/secyr/8/np.pi/sb/(r0*ld)**3 * (1. - np.sqrt(rinld/r0in))
 
 tv0out2 = np.sqrt(tv0out4)
 tv0out  = np.sqrt(tv0out2)
 return(tv0out)
 
 
 
def ti0(em,emdot,r0=1,hxsch=3,eta = 0.1):
 sb      = 5.670367e-8
 gnewt   = 6.67408e-11
 msun    = 1.98855e30
 secyr   = 31557600.0
 ld      = 2.59020683712e+13
 c       = 299792458.0
 hxld    = hxsch * 2*gnewt*msun*em/c/c/ld
 
 d2_ld    = hxld*hxld + r0*r0
 d1ld     = np.sqrt(d2_ld)
 d3ld     = d2_ld*d1ld
 ti0out4   = eta*msun/secyr*emdot*c*c/8/np.pi/sb * hxld/d3ld / ld/ ld
 ti0out2   = np.sqrt(ti0out4)
 ti0out    = np.sqrt(ti0out2)
 return(ti0out) 
 
 
 
 
 
#input grid r[....nr]] in light days, output t[r]
def tr(r,t0v,t0i,embh, r0=1,alpha_visc=-0.75,alpha_irad=-0.75,rinsch = 3):

 t0i2 = t0i*t0i
 t0i4 = t0i2*t0i2
 
 t0v2 = t0v*t0v
 t0v4 = t0v2*t0v2
 
 av4 = alpha_visc*4
 
 ai4 = alpha_irad*4
 
 rsch = 1.15821e-10*embh 
 rinld = rinsch*rsch
  
  
 if (rinld > r0):
  r0in = np.ceil(rinld/r0)*r0
  print('mytemp0.py inner radius bigger than reference radius (Mbh too big)  changing reference radius')
 else:
  r0in = r0
  
 #print r.shape
 #print np.mean(r)
 #print t0v4*(r)**av4
 
 tv4 = t0v4*(r/r0)**av4 * (1 - np.sqrt(rinld/r)) / (1 - np.sqrt(rinld/r0in))
 ti4 = t0i4*(r/r0)**ai4
 
 tout2 = np.sqrt(tv4 + ti4)
 tout  = np.sqrt(tout2)
 return(tout)

 
 







#rsinc ds207@lucky2.st-andrews.ac.uk:/microlens/ds207/fortcode/cream_export/katelc_group_1febmerge_* ./sd_hilag_111117/
import numpy as np
import glob
import matplotlib.pylab as plt
import os
import mymedquart as mm
import myrandom as mr
import myedlum as me
from astropy import constants as con
from astropy import units as u
import scipy.optimize

recalc = 1
plotmoddat = 1
plotedrat = 1
#tplotlim = 
logscal = [1,0]
xpl = [[2,99],[0.1,99.9]]
#xpl = [[1.e-5,10000.],[0.,1.]]
#ypl=[[0.,10.],[0.,1.]]

loglbol=0

labsave = ['mdot','cosinc']
axlab   = [r'$M\dot{M}$ $(10^7 M_\odot^2\mathrm{yr}^{-1})$',r'$\cos i$']



#make plots
#load data (if just redooing plot can skip to here)

parsave_plot = np.loadtxt('parsave_plot.txt')
parsigsave_plot = np.loadtxt('parsigsave_plot.txt')
lagsave_plot = np.loadtxt('lagsave_plot.txt')
lagsigsave_plot = np.loadtxt('lagsigsave_plot.txt')

lagsaveth_plot = np.loadtxt('lagsaveth_plot.txt')
lagsigsaveth_plot = np.loadtxt('lagsavesigth_plot.txt')




cisqred_plot = np.loadtxt('cisqred_plot.txt')
#fvar_plot = np.loadtxt('fvar_plot.txt')
snrstatsave_plot = np.loadtxt('snrstatsave_plot.txt')
fvm2_plot =  np.loadtxt('xsnorm_plot.txt')
fvm_plot = np.sqrt(fvm2_plot)
sigxs_plot = np.loadtxt('xs_plot.txt')

fname = 'snr_kate.txt'
with open(fname) as f:
    content = f.readlines()
# you may also want to remove whitespace characters like `\n` at the end of each line
content = [x.strip() for x in content] 
f.close()
snr_kate = [np.float(con.split()[1]) for con in content]


nsave,npar = np.shape(parsigsave_plot)


#if plotedrat on then load the black hole mass estimates for the sdss quasars, match the ids to the rm ids analysed by cream and calculate the eddington ratios
if (plotedrat == 1):
 os.system('python sdss_bhmasslist.py')
 datbh = np.loadtxt('sdss_bhmasslist.op')
 embhlog    = datbh[:,1]
 embhsiglog = datbh[:,2]
 rmid = datbh[:,0].astype(int)
 ummdot = parsave_plot[:,0]*1.e7
 ummdotsig = parsigsave_plot[:,0]*1.e7
 ummdotlog    = np.log10(ummdot)
 ummdotsiglog = 1./np.log(10)/ummdot * ummdotsig #np.log10(parsave_plot[:,0]*1.e7)

 #match rmid of cream runs to black hole masses in fits file
 targsave = np.loadtxt('targsave_op.txt')
 targload = datbh[:,0]
 idnow = np.array( [np.where(targload == tsnow)[0][0] for tsnow in targsave] )
 embhlog = embhlog[idnow]
 embhsiglog = embhsiglog[idnow]
 
 #calculate eddington ratios
 eta = 0.1
 a = np.array(me.edd2(embhlog, embhsiglog, ummdotlog, ummdotsiglog, eta)).T
 eddrat = a[:,2]
 sigeddrat = a[:,3]   
 
 
 #add a parameter to the parsave_plot column to instruct code to make extra plot
 a = np.ones((nsave,1))
 a[:,0] = a[:,0]*eddrat 
 parsave_plot = np.hstack((parsave_plot,a))
 a = np.ones((nsave,1))
 a[:,0] = a[:,0]*sigeddrat 
 parsigsave_plot = np.hstack((parsigsave_plot,a))
 labsave.append('eddrat')
 axlab.append(r'$L/L_\mathrm{edd}$')
 logscal.append(1)
 #xmed = np.percentile(eddrat,50)
 #xlo  = np.percentile(eddrat,1)
 #xhi  = np.percentile(eddrat,99)
 xpl.append([1,99])
 nsave,npar = np.shape(parsigsave_plot)


for ip in range(npar):
 fig = plt.figure()
 ax1 = fig.add_subplot(111)
 
 ynow = []
 xnow = []
 znow = []
 
 
 alphanow = 1/parsigsave_plot[:,ip]
 alphanow = alphanow/np.max(alphanow)
 for it in range(nsave):
  #xnow.append(cisqred_plot[it])
  ynow.append(snrstatsave_plot[it])
  xnow.append(parsave_plot[it,ip])
  znow.append(parsigsave_plot[it,ip])
  #znow.append(alphanow[it])
 
 xnow = np.array(xnow)
 ynow = np.array(ynow)
 znow = np.array(znow)
 idinc = np.arange(nsave)#np.where((xnow < xpl[ip][1]) &  (ynow < ypl[ip][1]))[0]
 xnow = xnow[idinc]
 ynow = ynow[idinc]
 znow = znow[idinc]
 nsnew = np.shape(xnow)[0] 
 #for it in range(nsnew): 
 #ax1.scatter(xnow,ynow,znow,color='b')
 ax1.scatter(xnow,ynow,color='b',s=3.0)
 #ax1.errorbar(xnow,ynow,xerr=znow,ls='',capsize = 10,color='k',alpha=0.3)

 #ax1.set_xlabel(r'$\chi^2/n$')
 ax1.set_xlabel(axlab[ip])
 ax1.set_ylabel('SNR = (Max flux - Min flux)/Median Error) \n averaged over all telscopes per filter')

 if (logscal[ip] == 1):
  ax1.set_xscale('log')
  ax1.set_yscale('log')
  #if (xpl[ip][0] != -1.0):
  # print 'xlim log plot',labsave[ip]+' ',xpl[ip]
  # ax1.set_xlim(xpl[ip])
 #elif (xpl[ip][0] == 0 & xpl[ip][1] == 0):
 print 'fsfsfd',xpl[ip]
 xlo = np.percentile(parsave_plot[:,ip],xpl[ip][0])
 xhi = np.percentile(parsave_plot[:,ip],xpl[ip][1])
 
 ax1.set_xlim([xlo,xhi])

 #else:
 # ax1.set_xlim(xpl[ip])
 # ax1.set_ylim(ypl[ip])
 plt.savefig('fig_'+labsave[ip]+'.pdf')






#make plot of snrstat (max(flux) - min(flux) / median errorbar) vs i - g continuum lag
try:
 idlaghi = 1
 idlaglo = 0
 fig = plt.figure()
 ax1 = fig.add_subplot(111)
 for it in range(nsave):
  x = lagsave_plot[it,idlaghi] - lagsave_plot[it,idlaglo]
  y = snrstatsave_plot[it]
  sigx = np.sqrt( lagsigsave_plot[it,idlaghi]**2 + lagsigsave_plot[it,idlaglo]**2 )
  ax1.errorbar(x,y,xerr=sigx,ls='',capsize = 10,color='k',alpha=0.3)
  ax1.scatter(x,y,color='k',s=0.5)
 
 ylim = list(ax1.get_ylim())
 ax1.plot([0.,0.],ylim,color='k',linewidth=3)
 ax1.set_ylim(ylim)
 ax1.set_xlabel(r'$\langle \tau_i \rangle - \langle \tau_g \rangle$ (days)')
 ax1.set_ylabel('SNR = (Max flux - Min flux)/Median Error) \n averaged over all telscopes per filter')
 plt.savefig('fig_snrplot.pdf')
except:
 print 'problem making lag plot fig_snrplot.pdf. Maybe these lcs were fitted with top hat contiuum lags'


#make plot of top hat continuum lags
idlaghi = 1
idlaglo = 0
fig = plt.figure()
ax1 = fig.add_subplot(211)
ax2 = fig.add_subplot(212)
xsave = []
ysave = []
for it in range(nsave):
 x = lagsaveth_plot[it]
 if (x[0] == 0):
  x = x[1] - x[0]
 elif (x[1] == 0):
  x = x[0] - x[1]

 
 y = snrstatsave_plot[it]
 xsave.append(x)
 ysave.append(y)
 sigx = np.sqrt( np.sum(lagsigsaveth_plot[it]**2)  )
 ax1.errorbar(x,y,xerr=sigx,ls='',capsize = 10,color='k',alpha=0.3)
 ax1.scatter(x,y,color='k',s=0.5)

xl = ax1.get_xlim()
ax2.hist(xsave,bins=np.linspace(xl[0],xl[1],60))
ylim = list(ax1.get_ylim())
ax1.plot([0.,0.],ylim,color='k',linewidth=3)
ax1.set_ylim(ylim)
ax1.set_xlabel(r'$\langle \tau_i \rangle - \langle \tau_g \rangle$ (days)')
ax1.set_ylabel('SNR = (Max flux - Min flux)/Median Error) \n averaged over all telscopes per filter')
ax2.set_xlabel(r'$\langle \tau_i \rangle$ (days)')
fig.tight_layout()
plt.savefig('fig_lagthsnrplot.pdf')



#make plot of top hat continuum lags
idlaghi = 1
idlaglo = 0
fig = plt.figure()
ax1 = fig.add_subplot(211)
ax2 = fig.add_subplot(212)
xsave = []
ysave = []
for it in range(nsave):
 x = lagsaveth_plot[it]
 if (x[0] == 0):
  x = x[1] - x[0]
 elif (x[1] == 0):
  x = x[0] - x[1]
 
 y = snr_kate[it]
 xsave.append(x)
 ysave.append(y)
 sigx = np.sqrt( np.sum(lagsigsaveth_plot[it])**2  )
 ax1.errorbar(x,y,xerr=sigx,ls='',capsize = 10,color='k',alpha=0.3)
 ax1.scatter(x,y,color='k',s=0.5)

ylim = list(ax1.get_ylim())
ax1.plot([0.,0.],ylim,color='k',linewidth=3)
ax1.set_ylim(ylim)
ax1.set_xlabel(r'$\langle \tau_i \rangle - \langle \tau_g \rangle$ (days)')
ax1.set_ylabel(r'SNR = RMS$/\sigma_\mathrm{RMS}$')
xl = ax1.get_xlim()
ax2.hist(xsave,bins=np.linspace(xl[0],xl[1],60))
ax2.set_xlabel(r'$\langle \tau_i \rangle$ (days)')
fig.tight_layout()
plt.savefig('fig_lagthrmsplot.pdf')

#make plot of snrstat (max(flux) - min(flux) / median errorbar) vs i - g continuum lag
idlaghi = 1
idlaglo = 0
fig = plt.figure()
ax1 = fig.add_subplot(111)
for it in range(nsave):
 x = lagsave_plot[it,idlaghi] - lagsave_plot[it,idlaglo]
 y = fvm_plot[it]
 sigx = np.sqrt( lagsigsave_plot[it,idlaghi]**2 + lagsigsave_plot[it,idlaglo]**2 )
 ax1.errorbar(x,y,xerr=sigx,ls='',capsize = 10,color='k',alpha=0.3)
 ax1.scatter(x,y,color='k',s=0.5)

ylim = list(ax1.get_ylim())
ax1.plot([0.,0.],ylim,color='k',linewidth=3)
ax1.set_ylim(ylim)
ax1.set_xlabel(r'$\langle \tau_i \rangle - \langle \tau_g \rangle$ (days)')
ax1.set_ylabel(r'$F_\mathrm{var}$ averaged over all telscopes per filter')
plt.savefig('fig_fvarplot.pdf')



#make plot of snrstat (max(flux) - min(flux) / median errorbar) vs i - g continuum lag
idlaghi = 1
idlaglo = 0
fig = plt.figure()
ax1 = fig.add_subplot(111)
for it in range(nsave):
 x = lagsave_plot[it,idlaghi] - lagsave_plot[it,idlaglo]
 y = sigxs_plot[it]
 sigx = np.sqrt( lagsigsave_plot[it,idlaghi]**2 + lagsigsave_plot[it,idlaglo]**2 )
 ax1.errorbar(x,y,xerr=sigx,ls='',capsize = 10,color='k',alpha=0.3)
 ax1.scatter(x,y,color='k',s=0.5)

ylim = list(ax1.get_ylim())
ax1.plot([0.,0.],ylim,color='k',linewidth=3)
ax1.set_ylim(ylim)
ax1.set_xlabel(r'$\langle \tau_i \rangle - \langle \tau_g \rangle$ (days)')
ax1.set_ylabel(r'$\sigma_\mathrm{XS}$ \n averaged over all telscopes per filter')
plt.savefig('fig_sigxs.pdf')


   
   
   
   
#make a plot of eddington ratio obtained from fits file bh mass bolometric correction
# vs cream value
eddrat_fits = datbh[idnow,10]
sigeddrat_fits = datbh[idnow,11]
ider = -1
eddrat_cream = parsave_plot[:,ider]
sigeddrat_cream = parsigsave_plot[:,ider]

fig = plt.figure()
ax1=fig.add_subplot(111)
ax1.errorbar(eddrat_fits,eddrat_cream,xerr=sigeddrat_fits,yerr=sigeddrat_cream,ls='',marker='',color='k',alpha=0.2)
ax1.scatter(eddrat_fits,eddrat_cream)
#ax1.set_xlim(np.percentile(eddrat_fits,[1,99]))
#ax1.set_ylim(np.percentile(eddrat_cream,[1,90]))
ax1.set_xlim([0,1])
ax1.set_ylim([0,1])

xl  = ax1.get_xlim()
yl = ax1.get_ylim()
ax1.plot(xl,yl,color='k',ls='--')
ax1.set_ylabel(r'$L/L_\mathrm{Edd}$ (CREAM)')
ax1.set_xlabel(r'$L/L_\mathrm{Edd}$ (Bolometric Correction)')
plt.savefig('fig_eddrat_comp.pdf')





fig = plt.figure()
ax1=fig.add_subplot(111)

y = eddrat_cream/eddrat_fits
idinc = np.where(y < np.inf)[0]
x = snrstatsave_plot
sigy = y * np.sqrt( (sigeddrat_cream/eddrat_cream)**2 + (sigeddrat_fits/eddrat_fits)**2 )
ax1.errorbar(x[idinc],y[idinc],sigy[idinc],color='k',alpha=0.3,ls='')
ax1.scatter(x[idinc],y[idinc],color='k',s=0.5)
#ax1.set_ylim([0,2])
ax1.set_yscale('log')
#ax1.set_ylim(np.percentile(eddrat_cream,[10,90]))
ax1.set_ylabel(r'$L/L_\mathrm{Edd}$ (CREAM) / $L/L_\mathrm{Edd}$ (fits)')
ax1.set_xlabel('SNR = (Max flux - Min flux)/Median Error) \n averaged over all telscopes per filter')
plt.savefig('fig_eddratrat_snr.pdf')




#ummdot
#ummdotsig 
em = 10**embhlog
emsig = np.log(10)*em*embhsiglog
eta = 0.1
mdot = ummdot/em
sigmdot = mdot * np.sqrt((emsig/em)**2 + (ummdotsig/ummdot)**2 )
syr = 31557600.0
conv = eta * con.c.value**2 * con.M_sun.value / syr * (1* u.joule).to(1* u.erg).value
lbol = mdot*conv
siglbol = sigmdot*conv 
x = datbh[idnow,12]
sigx = datbh[idnow,13]
#now plot Lbol vs nufnu





idgood = np.where((np.abs(np.log10(x)) < np.inf) & (np.abs(np.log10(x)) > 0))[0]
x = x[idgood]
sigx = sigx[idgood]



if (loglbol == 0):
 xlog = x
 sigxlog = sigx#1./np.log(10)/x * sigx
 xlogorth = np.mean(x)#np.mean(xlog)
 ylog = lbol[idgood]#np.log10(lbol)[idgood]
 sigylog = siglbol[idgood]#1./np.log(10)/lbol[idgood] * siglbol[idgood]
else:
 xlog = np.log10(x)
 sigxlog = 1./np.log(10)/x * sigx
 xlogorth = np.mean(xlog)
 ylog = np.log10(lbol)[idgood]
 sigylog = 1./np.log(10)/lbol[idgood] * siglbol[idgood]

 
def func(x,a,b): 
 return(a+b*x)
fit_coef=scipy.optimize.curve_fit(func,xlog-xlogorth,ylog,sigma=sigylog,bounds=[[-np.inf,-np.inf],[np.inf,np.inf]])
npar = np.shape(fit_coef[1])[0]
sig_coef = np.sqrt([[fit_coef[1][i,i]] for i in range(npar)])[:,0]
fit_coef = fit_coef[0]
xres = np.linspace(0.0,xlog.max(),500)#np.linspace(xlog.min(),xlog.max(),500)
yres = fit_coef[0] + (xres - xlogorth)*fit_coef[1]
yrlo = fit_coef[0] - sig_coef[0] + (xres - xlogorth)*(fit_coef[1] - sig_coef[1])
yrhi = fit_coef[0] + sig_coef[0] + (xres - xlogorth)*(fit_coef[1] + sig_coef[1])


fig = plt.figure()
ax1 = fig.add_subplot(111)
ax1.errorbar(xlog,ylog,xerr=sigxlog,yerr=sigylog,color='k',alpha=0.3,ls='')
ax1.scatter(xlog,ylog,color='k',s=0.5)
ax1.plot(xres,yres)
ax1.fill_between(xres,yrlo,yrhi,alpha=0.1)
if (loglbol == 1):
 ax1.set_ylabel(r'log $\eta \frac{M\dot{M}_\mathrm{CREAM}} {M_\mathrm{BH}} c^2$ (erg $\mathrm{s^{-1}}$)')
 ax1.set_xlabel(r'log $\nu F_\nu$ (erg cm$^{-2}$ $\mathrm{s^{-1}}$)')
else:
 ax1.set_ylabel(r'$\eta \frac{M\dot{M}_\mathrm{CREAM}} {M_\mathrm{BH}} c^2$ (erg $\mathrm{s^{-1}}$)')
 ax1.set_xlabel(r'$\nu F_\nu$ (erg cm$^{-2}$ $\mathrm{s^{-1}}$)')

#ax1.set_yscale('log')
#ax1.set_xscale('log')
ax1.set_ylim([0,1.e46])
ax1.set_xlim([0,1.4e46])
tit = r'$L_\mathrm{BOL} = ('+np.str(np.round(fit_coef[0],2))+'\pm '+np.str(np.round(sig_coef[0],2))+')$ $'+' \\nu L_\\nu$ (erg cm$^{-2}$ $\mathrm{s^{-1}}$)'
ax1.set_title(tit)
plt.savefig('fig_lbovsnufnu.pdf')




import numpy as np
import matplotlib.pylab as plt

#import myredcisq as mrc


twopi = 2*np.pi


#improved speed and added option to fit extra frequency components on top of drw
def fitrw(timeall,yall,sigall,floin=-1,fhiin=-1,ploton=0,plot_tit='fig_myrwfit',dtresin=-1,nits = 1,tplotlims=[],extra_f=[],
p0=-1,bpl = [0.5,2,2]): 

 w0 = bpl[0]*twopi

 parmout = []
 covout = []
 
 nlc = len(yall)
 ndatall    = []
 dtall   = []
 tmins = []
 tmaxs = []
 for i in range(nlc):
  ndat = np.shape(timeall[i])[0]
  ndatall.append(ndat)
  tmins.append(np.min(timeall[i]))
  tmaxs.append(np.max(timeall[i]))
  dtall.append(np.median(timeall[i][1:] - timeall[i][:-1]))
 

 
 tlo = np.min(tmins)
 thi = np.max(tmaxs)
 dt  = np.min(dtall)
 
 #input some minimum and maximum frequency range for the Fourier terms
 if (floin < 0):
  flo = 0.5/(thi-tlo)
 else:
  flo = floin
 
 if (fhiin < 0):
  fhi = 2.0/dt
 else:
  fhi = fhiin
 
 
 #angular frequency
 w = np.arange(flo,fhi+flo,flo)*2*np.pi
 
 dw = np.mean(w[1:] - w[:-1])
 #print flo,fhi,fhiin, 0.5/(thi-tlo)
 #w = np.arange(0,fhi+flo,5*flo)*2*np.pi
 #w[0] = flo/5000
 
 
 #prior
 if (bpl == []):
  w2 = w**2
  alpha = -2.
 else:
  bplin = list(bpl)
  #bplin[0] = twopi*bpl[0]
  #w2 = (1. + (w/(twopi*bplin[0]))**(bplin[1]-bplin[2])) / (w/(twopi*bplin[0]))**bplin[1]
 bplin = list(bpl)
 w2 = (1. + (w/(twopi*bplin[0]))**(bplin[1]-bplin[2])) / (w/(twopi*bplin[0]))**bplin[1]
 
 w_hi = w[-1]
 w_lo = w[0]
 w2_integrate_hi = w_hi*( (w_hi/(twopi*bplin[0]))**(-bplin[1])/(1.-bplin[1]) + (w_hi/(twopi*bplin[0]))**(-bplin[2])/(1.-bplin[2]) )
 w2_integrate_lo = w_lo*( (w_lo/(twopi*bplin[0]))**(-bplin[1])/(1.-bplin[1]) + (w_lo/(twopi*bplin[0]))**(-bplin[2])/(1.-bplin[2]) )
 w2_integrate = w2_integrate_hi - w2_integrate_lo
 
 
 
 #Mistake in algebra somewhere means we should square root the prior matrix (not sure why)
 w2 = w2
 
 #now compute the hessian matrix
 wold = np.array(w)
 nwold = np.shape(w)[0]
 np2old = 2*nwold
 
 
 #print 'this bit here'
 #the following has to be donw for as many light curves as you have
 
 nextra = len(extra_f)
 w_extra = np.array([ef*twopi for ef in extra_f])
 w = np.append(w,w_extra)
 nw = np.shape(w)[0]
 np2 = 2*nw
 for ilc in range(nlc):
  sig  = sigall[ilc]
  y    = yall[ilc]
  time = timeall[ilc]
  ndat = ndatall[ilc]
 
  #if p0 = -1 then use theoretical statistics and integration to set p0 
  #for random walk time series, rms = p0(whi^(alpha-1) - wlo(alpha-1)) / ((alpha+1) w0^alpha)
  rms = np.std(y)

  
  p0_out  = 2*rms/dw/w2_integrate#rms* ((alpha+1)*wold[0]**alpha / ( wold[-1]**(alpha+1) - wold[0]**(alpha+1)))
  
  
  #parameters for prior
  prior = 1.0#( wold[-1]**(alpha+1) - wold[0]**(alpha+1))/(alpha+1)/dw
  
  
  
  cvec = np.ones(np2+1)
  hes = np.ones((np2+1,np2+1))
  sig2 = sig*sig
  #save the sine and cosine value in  2d array
  cwt = np.ones((ndat,nw))
  swt = np.ones((ndat,nw))
  for iw in range(nw):
   cwt[:,iw] = np.cos(w[iw]*time)
   swt[:,iw] = np.sin(w[iw]*time)
  
  
  hnow_cc = np.tensordot(cwt.T/sig2,cwt,axes=1)#/sig2)
  hnow_sc = np.tensordot(swt.T/sig2,cwt,axes=1)#/sig2)
  
  hnow_cs = np.tensordot(cwt.T/sig2,swt,axes=1)#np.sum(cwt[:,iw]*swt[:,iwp]/sig2)
  hnow_ss = np.tensordot(swt.T/sig2,swt,axes=1)#np.sum(swt[:,iw]*swt[:,iwp]/sig2)
 
  
  #offset
  #cos
  iwp = 0
  hes[0,0] = np.sum(1./sig2)
  cvec[0] = np.sum(y/sig2)
  for ik in range(1,np2+1,2):
   hes[0,ik] = np.sum(cwt[:,iwp]/sig2)
   hes[0,ik+1] = np.sum(swt[:,iwp]/sig2)
   iwp = iwp + 1
  
  iwp = 0
  
  #print 'calculating cov matrix'
  
  
  
  
  for ikp in range(1,np2+1,2):
   wp = w[iwp]
   wp2 = wp*wp
   cvec[ikp]   = np.sum(y*cwt[:,iwp]/sig2)
   cvec[ikp+1] = np.sum(y*swt[:,iwp]/sig2)
   iw = 0
   #offset
   hes[ikp,0] = np.sum(cwt[:,iwp]/sig2)
   hes[ikp+1,0] = np.sum(swt[:,iwp]/sig2) 
   for ik in range(1,np2+1,2):
    #cosine column
    hes[ikp,ik]   = hnow_cc[iw,iwp]
    hes[ikp,ik+1] = hnow_sc[iw,iwp]
    if ((ik == ikp) & (ik <= np2old) ):
     hes[ikp,ik] = hnow_cc[iw,iwp] + w2_integrate/(2*rms*w2[iwp])#1./(p0*dw*w2[iwp])#*prior
    #sine column
    hnow_c = hnow_cs[iw,iwp]#np.sum(cwt[:,iw]*swt[:,iwp]/sig2)
    hnow_s = hnow_ss[iw,iwp]#np.sum(swt[:,iw]*swt[:,iwp]/sig2)
    hes[ikp+1,ik]   = hnow_cs[iw,iwp]
    hes[ikp+1,ik+1] = hnow_ss[iw,iwp]
    if ((ik == ikp) & (ik <= np2old)):
     hes[ikp+1,ik+1] = hnow_ss[iw,iwp] + w2_integrate/(2*rms*w2[iwp])
    
    iw = iw + 1  
   iwp = iwp + 1
        
  print('covariance')
  #invert the matrix to get the covariance matrix
  cov = np.linalg.inv(hes)
  
  
  #find the parameters
  parm  = cov.dot(cvec)
  skout = parm[2::2]
  ckout = parm[1::2]
  osout = parm[0]
  
  
  
  
  var = np.diag(cov)
  sd  = np.sqrt(var)#/1.6
  sdos  = sd[0]
  sdcos = sd[1::2]
  sdsin = sd[2::2]
  
  
  
  parmout.append(parm)
  covout.append(cov)
 
  freq = w/2/np.pi
 
 
  t_ext  = 0.0#0.2*(thi - tlo)
 
  if (dtresin == 0):
   dtres = dt/10
   tplot = np.arange(tlo-t_ext,thi+t_ext+dtres,dtres)
  elif (dtresin == -1):
   tplot = 1.*timeall[ilc]
  elif (isinstance(dtresin, np.ndarray) == 1):
   tplot = 1.*dtresin
   dtres = tplot[1] - tplot[0]
  else:
   dtres = dtresin
   tplot = np.arange(tlo-t_ext,thi+t_ext+dtres,dtres)
   
  
  if (tplotlims != []):
   tplot = np.arange(tplotlims[0],tplotlims[1],tplotlims[2])
   
  ntplot = np.shape(tplot)[0]
  xplot = 0*tplot
  #if (ploton == 1):
  #!!!!!!!!!!!!!!!!!!!!!!! plot the results
 
  
  xplot = []
  
  #resample using errors to get error snake
  #nits = 10
  xplotsave = np.zeros((ntplot,nits))
  xplot0_save = np.zeros((ntplot,nits))
  pspecsave = np.zeros((nw,nits))
  
  
  print('calculating error sake...')
  #draw paraeters from multivariate gaussian
  
  #need to limit the size of the parameters to sample 
  # with multivariate gaussian otherwise will take forever
  #include max of 2000 plus the extra frequencies
  ipmax = min(500,np2+1)
  #sensibly sort the covariance matrix to identify the most correlated parameteres
  #for multivariate gaussian stepping
  cor = np.abs(np.corrcoef(cov))
  corabs_sum = np.sum(np.abs(cor),axis=0)/(np2+1)
  ipinc = np.argsort(corabs_sum)[-1::-1]
  #print corabs_sum[ipinc][:10]
  #print 'average correlation coeff (abs) for mvg'
  #print 'corabssum[:10]'
  #print corabs_sum[ipinc][:min(10,np2+1)]
  #print ''
  #
  #print 'corabssum[-10:]'
  #print corabs_sum[ipinc][max(0,np2+1-10):]
  #print ''
  #
  #print 'corabssum chop'
  #print corabs_sum[ipinc][ipmax-1]
  #print ''
  
  ipinc = ipinc[:ipmax]
  #idsort = np.dstack(np.unravel_index(np.argsort(cor.ravel()), (np2old+1, np2old+1)))
  
  
  #ipinc = np.append(np.arange(ipmax),np.arange(np2old+1,np2+1))
  ipdel = np.array([i for i in range(np2+1) if i not in ipinc])
  #ipdel = np.arange(ipmax,np2old+1,1)
  psd = np.sqrt(np.diag(cov))
  parmnew = 1.*parm
  if (nits > 1):
   covnew = np.delete(cov,ipdel,axis=0)
   covnew = np.delete(covnew,ipdel,axis=1)
   print(np.shape(covnew),np.shape(parm),np.shape(parm[ipinc]))
   parmnow = np.random.multivariate_normal(parm[ipinc],covnew,size=nits)
   
  for i in range(nits):
   print('errorbars',i,nits)
   
   #cknew = normdis(1,ckout,sdcos)
   #sknew = normdis(1,skout,sdsin)
   #osnew = normdis(1,osout,sdos)
   #need to use multivariate gaussian to sample correlated probability space
   
   if (nits > 1):
    parmnew = parm + np.random.randn(np2+1)*psd
    parmnew[ipinc] = parmnow[i,:]
   
   osnew = parmnew[0]
   cknew = parmnew[1::2]
   sknew = parmnew[2::2]
   
   #cknew[0] = ckout[0]
   #sknew[0] = skout[0]
   print('osnew',osnew,parm[0])
   pspecsave[:,i] = sknew*sknew + cknew*cknew
   
   xplot0_save[:,i] = osnew# + cknew[0]*np.cos(w[0]*tplot) + sknew[0]*np.sin(w[0]*tplot)
   for it in range(ntplot):
    tnow = tplot[it]
    xplotsave[it,i] =  osnew + np.sum(cknew*np.cos(w*tnow) + sknew*np.sin(w*tnow))
    
  xplot = np.mean(xplotsave,axis=1)
  xplot0_sd = np.ones(ntplot)*sdos
  
  xplotsd = np.std(xplotsave,axis=1)# - xplot0_sd
  
  
  #plot errors for power spectrum also
  pspec = ckout**2 + skout**2
  pspecsd = np.std(pspecsave,axis=1)
  
  #plot the input and output light curves to see if they match
  xplot0 = np.ones(ntplot)*osout#ckout[0]*np.cos(w[0]*tplot) + skout[0]*np.sin(w[0]*tplot)
  #for tnow in tplot:
  # xplot.append( np.sum(ckout*np.cos(w*tnow) + skout*np.sin(w*tnow)) )
  if (ploton == 1):
   fig = plt.figure()
   ax1 = fig.add_subplot(211) 
   ax1.errorbar(time,y,sig,ls='',color='k')
   ax1.plot(tplot,xplot0,color='b',ls='--')
   ax1.plot(tplot,xplot0-xplot0_sd,color='b',ls='--')
   ax1.plot(tplot,xplot0+xplot0_sd,color='b',ls='--')
   
   
   
   #calculate the reduced chi squared
   #print 'chisq info', mrc.myredcisq(time,y,sig,tplot,xplot,npar = 0)
   
   
   
   #for i in range(nits):
   # ax1.plot(tplot,xplotsave[:,i],color='b',alpha=0.06)
   ax1.fill_between(tplot,xplot-xplotsd,xplot+xplotsd,alpha=0.2,color='k',label=None)
   
   ax1.plot(tplot,xplot,color='b')
   ax1.set_xlabel('Time (days)')
   ax1.set_ylabel('flux')
   ylim = [np.min(y),np.max(y)]
   ax1.set_ylim(ylim)
   ax2 = fig.add_subplot(212) 
   
   ax2.plot(freq,pspec,ls='',marker='o',color='k')
   ax2.fill_between(freq,pspec-pspecsd,pspec+pspecsd,color='k',alpha=0.2)
   #ax2.plot(freq,sdsin,ls='',color='g',marker='o')
   #ax2.plot(freq,sdcos,ls='',color='b',marker='o')
   ax2.set_xscale('log')
   ax2.set_yscale('log')
   ax2.set_xlabel('Frequency (cyc/day)')
   ax2.set_ylabel(r'power ($S_k^2 + C_k^2$)')
   ax2.grid(True)
   
   if (plot_tit == ''):
    plt.show()
   else:
    plt.savefig(plot_tit+'_'+np.str(ilc)+'.png')
   
 sig2_prior = (2*rms*w2)/w2_integrate
 return(parmout,covout,freq,tplot,xplot,xplotsd,p0_out,w0,dw,sig2_prior)










#!!!!!!!!!!!!!!!!!!! test the code using this program

##
#from mylcgen import *
#from myrandom import *
#import myresample as mrs
#timeall = []
#sigall  = []
#yall    = []
#
#nlc = 1
#
#
#
#for i in range(nlc):
# #generate test light curve and add noise
# datpre      = mylcgen(tlo=0,thi=100,dt=0.03,iseed=132423)
# npre     = np.shape(datpre[:,0])[0]
# datmean  = np.std(datpre[:,1])
# snow =  np.ones(npre)/10*datmean 
# dat = mrs.myresample(dir='',fname=[''],dtave=1.0,sampmin=0.8,sampcode=3,datin=np.array((datpre[:,0],datpre[:,1],snow)).T)
# ndat = np.shape(dat[:,0])[0]
# sig = dat[:,2]
# for i in range(ndat):
#  dat[i,1] = normdis(1,dat[i,1],sig[i])[0]
# sigall.append( sig )
# yall.append( dat[:,1] +50 )
# timeall.append( dat[:,0] )
# 
# 
#print 'going in'
#a = fitrw(timeall,yall,sigall,ploton=1,dtresin = 0.1,nits = 100,extra_f=[]) 




    


import numpy as np


import numpy as np
import my_kateccf as mccf
import mylcgen as mlc
import matplotlib.gridspec as gridspec
import matplotlib.pylab as plt
import scipy.signal as ss

ilag = 5
sigma = 0.3

#generate synthetic data
drive = mlc.mylcgen(datfile='',p0=1.0,f0=0.1,a=-2,b=-2,tlo=0,thi=100,dt=1.0,ploton=0,iseed=-1,meannorm = -1., sdnorm = -1.0)
 
t = drive[:,0]
x = drive[:,1]
xsd = np.std(x)
nx = np.shape(x)[0] 
x = (x - np.mean(x))/xsd
xsd = np.std(x)
sigx = np.ones(nx)*xsd*sigma
x = x + np.random.randn(nx)*sigx

lc1 = np.array([t,x,sigx]).T

#define convolution kernel
conv = np.zeros(nx)
conv[ilag-1:ilag+1] = 1



echo  = np.convolve(x, conv, mode='same')
echo2 = np.convolve(x, conv, mode='full')[:nx]
 
echo2 = (echo2 - np.mean(echo2))/np.std(echo2) 
esd = np.std(echo2)
ne2=np.shape(echo2)[0]
sige = np.zeros(ne2)+esd*sigma
echo2 = echo2 + np.random.randn(ne2)*esd*sigma
lc2 = np.array([t,echo2,sige]).T




frac = 0.8

dt = 0.1
tmin = min(np.min(lc1[:,0]),np.min(lc2[:,0]))
tmax = max(np.max(lc1[:,0]),np.max(lc2[:,0]))

n1 = np.shape(lc1[:,0])[0]
n2 = np.shape(lc2[:,0])[0]
#interpolate time series onto regular sampled grid
tgrid = np.arange(tmin.tmax,dt)
ngrid = np.shape(tgrid)[0]
id1 = np.arange(n1)
id2 = np.arange(n2)


nsub_1 = np.int(frac*n1)
nsub_2 = np.int(frac*n2)



for iteration in range(nits):
 
 idsub1 = np.random.choice(id1,size = nsub_1)#replace true by default
 idsub2 = np.random.choice(id2,size = nsub_2)
 
 y1_itp = np.interp(tgrid,lc1[idsub1,0],lc1[idsub1,1])
 y2_itp = np.interp(tgrid,lc2[idsub2,0],lc2[idsub2,1])

 ccf = ss.correlate(x,echo2)
 nccf = np.shape(ccf)[0]
 tccf = (np.arange(nccf) - nccf/2)*dt














import numpy as np
import sys
import pandas as pd
import os


def filt_id(fnow):
 #identify wavelength
 if (fnow == 'B'):
  wav = 4392.0
 elif (fnow == 'V'):
  wav = 5468.0
 elif (fnow == 'HX'):
  wav = 4.4
 elif (fnow == 'SX'):
  wav = 25.3
 elif (fnow == 'U'):
  wav = 3465.0
 elif (fnow == 'UVM2'):
  wav = 2246.0
 elif (fnow == 'UVW1'):
  wav = 2600.0
 elif (fnow == 'UVW2'):
  wav = 1928.0
 
 else:
  wav = 0.0
  print 'unknown wavelength filter:', fnow
  print 'aborting...'
  sys.exit()
 
 return(wav)









path = '/Users/ds207/Documents/standrews/sta/lightcurves/rick_newlc_feb2018'
filename = 'swift4agn.xlsx'#'nearfinal.xlsx'

dirsave = path


df = pd.read_excel(path + '/' + filename)
print df.head()





#identify object colun
o=df[u'Object']
f=df[u'Filter']

ounique = list(set(o))
funique = list(set(f))
wavunique = [filt_id(fn) for fn in funique]
idwavsort = np.argsort(wavunique)
funique = [funique[idw] for idw in idwavsort]

for onow in ounique:
 dsave = dirsave+'/'+onow
 os.system('rm -rf '+dsave)
 os.mkdir(dsave)
 
 
 
 #make creamnames file
 f = open(dsave+'/creamnames.dat','w')
 
 for fnow in funique:
  
  wav = filt_id(fnow)
  
  

  
  
  a = df.loc[(df[u'Object']==onow) & (df[u'Filter']==fnow),['MJD','Flux','Error']]
  asort = a.sort_values(['MJD'],ascending=True)
  asort = asort.round(9)
  fname = onow+'_'+fnow+'.dat'
  asort.to_csv(dsave+'/'+fname,header=False,index=False,sep=' ')
  
  f.write('"'+fname+'" '+np.str(wav)+'\n')
  
 f.close()
  
  
  
#iobject = 0
#ifilter = 1
#itime   = 3
#iflux   = 4
#isig    = 5
#
#
#
#
##df = pd.read_excel(path + '/' + filename,columns=['object','filter','cadence','mjd','flux','error','coad'])
##print df.head()
#
#
#with open(path+'/'+filename) as f:
#    content = f.readlines()
## you may also want to remove whitespace characters like `\n` at the end of each line
#content = [x.strip() for x in content] 
#nc = len(content)
#f.close()
#
#
#for i in range(nc):
# fnow = content[i].split(',')
# 



import numpy as np
import glob

#read the creanames file and load wavelengths and files
def read_creamnames(dir,file='creamnames.dat'):

 f = open('../../creamnames.dat')
 filedat = f.readlines()
 filedat = [i.strip() for i in filedat]#remove \n
 wav     = [i.split(' ')[-1] for i in filedat]
 wav = [float(x) for x in wav]

 return(filedat,wav)
 
 
 
 

def cream_merge(dirin,filecontref=0,opfile_tf='',file_lc='./plots/modellc_merge.dat',file_lcsig='./plots/modellc_sig_merge.dat',opfile_tf='./plots/modeltf_merge.dat',file_tfsig='./plots/modeltf_sig_merge.dat',opfile_filewav='./outputpars3_merge.dat',file_osst='./outputpars2_merge.dat',file_fileth='./outputpars_th_merge.dat',file_ve='./outputpars_varexpand_merge.dat',file_creamnames='../creamneames_merge.dat'):
 
#cream_merge(dir,wav='',
#file='',
#opfile_tf=''
#,file_lc='./plots/modellc_merge.dat',
#file_lcsig='./plots/modellc_sig_merge.dat',
#opfile_tf='./plots/modeltf_merge.dat',
#file_tfsig='./plots/modeltf_sig_merge.dat',
#opfile_filewav='./outputpars3_merge.dat',
#file_osst='./outputpars2_merge.dat',
#file_fileth='./outputpars_th_merge.dat',
#file_ve='./outputpars_varexpand_merge.dat',
#file_creamnames='../creamneames_merge.dat'
  
 if 'output_20' in dirin:
  dir = dirin  
 else:
  print 'output file not in dir string...',dirin
  dir = glob.glob(dirin+'./output_20*')[-1]
  print 'looking here...',dir
  
   


 #input files are just outputfiles without the _merge.dat
 #load input files
 filelc,wav=read_creamnames(dir+'/../creamnames.dat')
 nlc = len(wav)
 wavu = np.unique(wav)
 nwavu = np.shape(wavu)[0]
 idwavu = [np.where(wav == wavu[iu])[0] for iu in range(nwavu)]
 idth = np.where(wavu == -1)[0]
 wavu = wavu[np.where(wavu != -1)[0]]
 nth  = np.shape(idth)[0]

 
 nop = nth + wavu
 
 datlc = np.loadtxt(file_lc)
 datlc_sig = np.loadtxt(file_lcsig)
 dattf = np.loadtxt(file_tf)
 dattf_sig = np.loadtxt(file_tfsig)
 
 ntgrid = np.shape(datlc[:,0])[0]
 ntaugrid = np.shape(dattf)[0]
 datlc_op = np.zeros((ntgrid,nop+1))
 dattf_op = np.zeros((ntaugrid,nop+1))
 
 tgrid = datlc[:,0]
 taugrid = dattf[:,0]
 
 
 #identify which continuum ight curve to use as the reference when merging
 idcont = []
 if (isinstance(filecontref,list)):
  for iu in range(nwavu):
   idcont.append([i for filelc[i] in range(nlc) if filecontref[iu] in filelc[i]][0])
 else:
  idcont.append(wavu[) 
 
 
 #model light curves: compute the median and standard deviation
 idxinc = [np.where((datlc[:,1+ilc] != 0))[0] for ilc in range(nlc)]
 median = [np.median(datlc[idxinc[ilc],1+ilc]) for ilc in range(nlc)]
 std    = [np.std(datlc[idxinc[ilc],1+ilc]) for ilc in range(nlc)]

 
 #populate the model arrays for the continuum wavelength
 for iw in range(nwavu):
  wavunow = wavu[iw]
  inow = np.where(wav == wavunow)[0]
  
  
  
  
  
  
  
  
  datlc_op[:,0] = tgrid
  datlc_op[:,iw] = inow[
  
  #light curves
  
 
 #if output_20xxx in dir string then look in there else look for subdir in dir string
 



#!!!! Extracts shorter light curve between two times and dumps it to another text file. 
#!! See below for arguments

import numpy as np
import pickle
import glob
import os
import sys
from pylab import *

def mypicktimes(dir,lines,tloold,thiold,outfile='creamnames.dat',plotop=0,submin=0):
 #dir     = sys.argv[1] #the directory storing the light curves e.g '../fort/fortcode/fakelc/kep_18mar'
 #lines   = sys.argv[2] # a list of the files .e.g ['file1.dat','file2.dat','...'] etc
 #dtave   = sys.argv[3] #e.g 0.5 will resample with mean half day cadence
 #tloold  = sys.argv[4] #lower time
 #thiold  = sys.argv[5] #upper time
 #outfile = sys.argv[6] #name of file storing the names of the outputted light curves
 
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! arguments above
 #tloold  = 241.0
 #thiold  = 341.0
 ntnew   = thiold - tloold + 1
 toffset = 0.0
 ioffset = [2] #this is the kepler light curve where we apply the offset
 
 pwd    = os.getcwd()
 #dir    = './kep_18mar'#./fake_backups_CREAM_paper'
 #fnames = 'mypicktimesdat.dat'
 #snrfile = 'addnoise_snr.dat'
 #outfile = 'mcmcmultinames.dat'
 #dirs = 'default'
 
 #with open(fnames) as f:
 #    lines = f.read().splitlines()
 
 
 os.chdir(dir)
 
 #dirprev = glob.glob('*'+dirs+'*')
 #nprev = len(dirprev)
 #dirnew = dirs+'_'+str(nprev+1)
 #os.mkdir(dirnew)
 #pwd = os.getcwd()
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! read files lcfile and snr file
 #  
 
 
     
 nf = len(lines)
 
 fnamenew=[]
 for i in range(nf):
  dat = np.loadtxt(lines[i])
  
  
  if (i in ioffset):
   tlo = tloold
   thi = thiold
  else:
   tlo = tloold + toffset
   thi = thiold + toffset
  
  t = dat[:,0]
  print np.min(t),np.max(t),tlo,thi
  idxlo = np.where(t > tlo)[0][0]
  
  nhi   = np.where(t > thi)[0].shape[0]
  if (nhi > 0):
   idxhi = np.where(t > thi)[0][0]-1 
  else:
   idxhi = t.shape[0] - 1
  
  ntnew = idxhi - idxlo
  datnew = np.zeros((ntnew,3)) 
  datnew[:,0] = dat[idxlo:idxhi,0]
  datnew[:,1] = dat[idxlo:idxhi,1]
  datnew[:,2] = dat[idxlo:idxhi,2] 
  
  
  if (submin == 1):
   datnew_min = np.min(datnew[:,0])
   datnew[:,0] = datnew[:,0] - datnew_min
   
  if (plotop == 1):
   errorbar(datnew[:,0],datnew[:,1],datnew[:,2],ls='')
   show()
   
  
  fnamenew.append( 'length_'+lines[i] )
  
  #os.chdir(dirnew)
  np.savetxt(fnamenew[i],datnew)
  #os.chdir(pwd)
  #os.chdir(dir)
  
  
  
 #write to file
 #os.chdir(dirnew)
 of = open(outfile,'w') 
 for item in fnamenew:
   of.write("%s\n" % item)
  
 of.close()
 os.chdir(pwd)
 return()


import numpy as np
import pandas as pd


#input dataframe with time axis as datetime object,
#2nd column should be the y values of the timeseries
#noise argument assigns error bars as a function of 
#light curve standard deviation (I dont know if this
#is allowed but should be fine)

def convert_df_2_lightcurve(df,reftime='auto',noise = 0.1,
normalise = 1):
 
 
 #print(df.iloc[:10,0])
 #print(df.iloc[:10,1])
 #input()
 ny,nalong = np.shape(df.values)
 
 
 if (reftime == 'auto'):
  print('setting reference time as the earliest elemnt in the time axis in day units')
  t0 = pd.Timestamp(year=2015,month=1,day=1)
 else:
  t0 = reftime 
  
 datemin = pd.Timestamp(year=2015,month=1,day=1)
 #try:
 
 cols = df.columns[0]
 dfsort = df.sort_values(by=cols)
 
 

 
 timesort = dfsort.iloc[:,0]
 datsort  = dfsort.iloc[:,1]
 
 #df = df.sort_index(axis=0)
 print(type(timesort),type(t0))
 dt = timesort - t0
 
 #t  = np.array([dtn.days() for dtn in dt])
 #something wrong above. Manually convert to days instead
 dt = np.array(dt,dtype='timedelta64[s]').astype('timedelta64[D]')
 t = dt.astype('float')

 #for i in range(ny):
 # print(t[i],df.iloc[i,1])
 y  = np.array(datsort)
 ystd = np.std(y)
 
 #if dataframe has 3 columns then one is the errors and we dont have to manufacture these
 if (nalong > 2):
  ysig = df.values[:,2]
 else:
  ysig = np.zeros(ny) + noise*ystd
 

 if (normalise == 1):
  y = (y - np.mean(y))/ystd
  ysig = ysig/ystd
 
 
  
#except:
#  raise Exception('first column must be time axis in datetime format')
  
  
 #return ny X 3 array with ny columns and rows corresponding to time, y, errorbar
 return( np.array([t,y,ysig]).T )
 
 
 


from setuptools import setup

#pip install .
#python setup.py sdist
#twine upload dist/astropy_stark-1.1.8.tar.gz
setup(name='astropy_stark',
      version='1.1.8',
      description='random-custom astronomy functions including making fake accretion disk light curves',
      url='https://github.com/dstarkey23/pycecream',
      author='dstarkey23',
      author_email='ds207@st-andrews.ac.uk',
      license='MIT',
      packages=['astropy_stark'],
      install_requires=[
      'scikit-learn',
      'scipy',
      'statsmodels',
      'pandas',
      'numpy',
      'matplotlib',
      'glob3',
      ],
      zip_safe=False)

#jupyter nbconvert --to Markdown ./pycecream_test/test_pycecream.ipynb


"""
Version 1, by Mouyuan (Eric) Sun
email: ericsun88@live.com
Version 2, by Kate Grier
email: catherine.grier@gmail.com

Copyright (c) 2018 Mouyuan Sun and Catherine Grier; catherine.grier@gmail.com  

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.


####################################
This code is meant to emulate a fortran program written by B. Peterson,
which cross correlates two light curves that are unevenly sampled using linear
interpolation and measure sthe peak and centroid of the cross-correlation function.
In addition, it is possible to run Monteo Carlo iterationsusing flux randomization
and random subset selection (RSS) to produce cross-correlation centroid distributions to
estimate the uncertainties in the cross correlation results.

The idea is described in detail in this work:
Peterson et al.(1998): http://arxiv.org/abs/astro-ph/9802103

The modules included are:

1. corsig: Calculates the p-value for cross-correlation (called by peakcent and xcor_mc). For
most purposes, this does not need to be used -- it is called by modules 3 and 4. 

2. xcor: Calculates the cross correlation function (called by peakcent and xcor_mc). Again does not
need to be used -- is called by the below two modules. 

**3. peakcent: Calls modues 1 and 2 to determine the peak and centroid of the cross correlation
function between two light curves, and returns the peak, centroid, CCF, r_max value,
and additional information.

**4. xcor_mc: Runs the Monte Carlo iterations using flux randomization (FR) and/or random
subset selection (RSS). Calls all of the above modules. Returns the cross correlation
centroid distribution (CCCD) and peak distribution (CCPD) of time lags as well as
information on failed calculations. 

####################################
Version history:
Version 1.0 (May 27, 2015)
Version 1.1 (Nov 15, 2015):
Bug fix: make sure the peak is significant when calculate the centroid
         (i.e., add ' and status_peak==1 ' to line 212)
Version 2.0: (Jan 5, 2018)
This version of the software hasbeen edited fairly extensively by Kate Grier and Jennifer Li
from its original version, to fix bugs and add additional functionality.

#####################################
"""

import numpy as np
import scipy.stats as sst
from matplotlib import pyplot as plt



def corsig(r, v):
    '''
    Calculate the p value that a random uncorrelated sample can yield a 
    correlation coefficient as large as or larger than the observed absolute 
    value of r, where r is the correlation coefficient of the data (using 
    t test, valid if v>=4)
    Ref1: http://janda.org/c10/Lectures/topic06/L24-significanceR.htm
    Ref2: http://vassarstats.net/textbook/ch4apx.html
    
    Inputs:
        r -- the correlation coefficient of the data
        v -- degree of freedom when calculating r: N-2 (hence N>2!!!)
    Outputs:
        pvalue
    '''
    r = float(r)
    v = float(v)
    
    r2 = r*r
    tst = r*np.sqrt(v/(1-r2))
    pvalue = sst.t.sf(tst, v) # sf: survival function -- 1-CDF
    return pvalue
    



def xcor(t1, y1, t2, y2, tlagmin, tlagmax, tunit, imode=0):
    '''
    Calculate cross-correlation function for unevenly 
    sampling data.
    
    Inputs:
        t1 -- time for light curve 1, assume increase;
        y1 -- flux for light curve 1;
        t2 -- time for light curve 2, assume increase;
        y2 -- flux for light curve 2;
        tlagmin -- minimum time lag;
        tlagmax -- maximum time lag;
        tunit -- tau step;
        imode -- cross-correlation mode: 0, twice (default); 
                 1, interpolate light curve 1;
                 2, interpolate light curve 2.
        
    Outputs:
        ccf -- correlation coefficient;
        tlag -- time lag (t2 - t1); positive values mean second
                  light curve lags the first light curve, as per convention.
                 (edit by kate, march 2016)
        npts -- number of data points used;
    '''
    safe = tunit*0.1
    taulist = []
    npts = []
    ccf12 = []  # interpolate 2
    ccf21 = []  # interpolate 1
    
    # frist interpolate 2
    tau = tlagmin + 0.0
    while tau < tlagmax+safe:
        t2new = t1 - tau
        selin = np.where((t2new>=np.min(t2))&(t2new<=np.max(t2)), True, False)
        knot = np.sum(selin)  # number of datapoints used
        if knot>0:
            y2new = np.interp(t2new[selin], t2, y2)
            
            y1sum = np.sum(y1[selin])
            y1sqsum = np.sum(y1[selin]*y1[selin])
            y2sum = np.sum(y2new)
            y2sqsum = np.sum(y2new*y2new)
            y1y2sum = np.sum(y1[selin]*y2new)
            
            fn = float(knot)
            rd1_sq = fn*y2sqsum - y2sum*y2sum
            rd2_sq = fn*y1sqsum - y1sum*y1sum
            if rd1_sq>0.0:
                rd1 = np.sqrt(rd1_sq)
            else:
                rd1 = 0.0
            if rd2_sq>0.0:
                rd2 = np.sqrt(rd2_sq)
            else:
                rd2 = 0.0
            
            if rd1*rd2==0.0:
                r = 0.0
            else:
                r = (fn*y1y2sum - y2sum*y1sum)/(rd1*rd2)
            ccf12.append(r)
            taulist.append(tau)
            npts.append(knot)
        tau += tunit
    # now interpolate 1
    tau = tlagmin + 0.0
    while tau < tlagmax+safe:
        t1new = t2 + tau
        selin = np.where((t1new>=np.min(t1))&(t1new<=np.max(t1)), True, False)
        knot = np.sum(selin)  # number of datapoints used
        if knot>0:
            y1new = np.interp(t1new[selin], t1, y1)
            
            y2sum = np.sum(y2[selin])
            y2sqsum = np.sum(y2[selin]*y2[selin])
            y1sum = np.sum(y1new)
            y1sqsum = np.sum(y1new*y1new)
            y1y2sum = np.sum(y1new*y2[selin])
            
            fn = float(knot)
            rd1_sq = fn*y2sqsum - y2sum*y2sum
            rd2_sq = fn*y1sqsum - y1sum*y1sum
            if rd1_sq>0.0:
                rd1 = np.sqrt(rd1_sq)
            else:
                rd1 = 0.0
            if rd2_sq>0.0:
                rd2 = np.sqrt(rd2_sq)
            else:
                rd2 = 0.0
            
            if rd1*rd2==0.0:
                r = 0.0
            else:
                r = (fn*y1y2sum - y2sum*y1sum)/(rd1*rd2)
            ccf21.append(r)
        tau += tunit
    
    # return results according to imode
    taulist = np.asarray(taulist)
    npts = np.asarray(npts)
    ccf12 = np.asarray(ccf12)
    ccf21 = np.asarray(ccf21)
    if imode==0:
        ccf = (ccf12 + ccf21)*0.5
    elif imode==1:
        ccf = ccf21 + 0.0
    else:
        ccf = ccf12 + 0.0
    
    return ccf, -taulist, npts




def peakcent(t1, y1, t2, y2, tlagmin, tlagmax, tunit, thres=0.8, siglevel=0.95, imode=0, sigmode = 0.2):
    '''
    Calculate peak time lag and centroid based on the cross-correlation 
    function for unevenly sampling data.
    
    Inputs:
        t1 -- time for light curve 1, assume increase;
        y1 -- flux for light curve 1;
        t2 -- time for light curve 2, assume increase;
        y2 -- flux for light curve 2;
        tlagmin -- minimum time lag;
        tlagmax -- maximum time lag;
        tunit -- tau step;
        thres -- lower limit of correlation coefficient when 
                 calculate centroid, default is 0.8;
        siglevel -- the required significant level of the 
                 correlation coefficient;
        imode -- cross-correlation mode: 0, twice (default); 
                 1, interpolate light curve 1;
                 2, interpolate light curve 2.
        sigmode -- how to deal with significance:
                Will use r = input value as the minimum correlation coefficient to consider (default = 0.2).
                0: Will use a p-test to assign significance to peak and discard peaks that are below
                the significance threshold (depends on number of points included and r). 
        
    Outputs:
        tlag_peak -- time lag based on the peak argument;
        status_peak -- peak status (1, constrained; 0, unconstrained);
        tlag_centroid -- time lag for centroid;
        status_centroid -- centroid status (1, constrained; 0, unconstrained);
    '''
    alpha = 1.0 - siglevel  # probability threshold to reject: no correlation hypothesis
    
    ccf_pack = xcor(t1, y1, t2, y2, tlagmin, tlagmax, tunit, imode)
    
    # ccf peaks --- excluding all with r < 0.2 instead of using p-value test. 
    if sigmode > 0:
        #print 'Using minimum r coefficient instead of significance test.'        
        max_indx = np.argmax(ccf_pack[0])
        max_rval = ccf_pack[0][max_indx]
        #Check and see if the max r is on the edge of the CCF. Fail it if so. 
            
        if np.logical_and(max_rval >= sigmode, np.logical_and(ccf_pack[1][max_indx] != tlagmin, ccf_pack[1][max_indx] != tlagmax)): 
            tlag_peak = ccf_pack[1][max_indx]
            max_rval = max_rval
            status_peak = 1
            status_rval = 1
        else:
            max_rval = -9999.0
            tlag_peak = -9999.0
            tlag_centroid = -9999.0
            status_peak = 0
            status_rval = 0
            status_centroid = 0

        #If the peak succeeds, calculate centroid:
        if status_peak == 1:
            rcent = thres*ccf_pack[0][max_indx]
            
            #populate arrays for centroid calculation!!
            lags_cent = []
            rval_cent = []
            temp_r = max_rval

            #start at r_max and lag_r_max
            temp_lag = ccf_pack[1][max_indx]
            temp_index = max_indx
            
            #Go higher in tau; if any of this fails, consider it a fail. 
            try:
                while temp_r >= rcent:
                    lags_cent.append(temp_lag)
                    rval_cent.append(temp_r) 

                    temp_index +=1
                    temp_r = ccf_pack[0][temp_index]
                    temp_lag = ccf_pack[1][temp_index]

                #Now start at the pixel to the left of the max
                temp_r = ccf_pack[0][max_indx-1]
                temp_lag = ccf_pack[1][max_indx-1]
                temp_index = max_indx-1
            
            #Now go downwards in lag
                while temp_r >= rcent:
                    lags_cent.append(temp_lag)
                    rval_cent.append(temp_r) 

                    temp_index -=1
                    temp_r = ccf_pack[0][temp_index]
                    temp_lag = ccf_pack[1][temp_index]

                lagr = zip(lags_cent, rval_cent)
                lagr.sort()
                lags_cent = [x for x, y in lagr]
                rval_cent = [x for y, x in lagr]
            
                tlag_centroid = np.sum(np.array(rval_cent)*np.array(lags_cent))/np.sum(np.array(rval_cent))
                status_centroid = 1                    
                
            except:
                tlag_centroid = -9999.0
                status_centroid = 0
                          
            #For debugging
            #plt.plot(ccf_pack[1], ccf_pack[0], color = 'k')
            #plt.plot(lags_cent, rval_cent, color = 'r')
            #plt.axhline(y = rcent, color = 'r', linestyle = ':')
            #plt.axhline(y = max_rval, color = 'r', linestyle = '--')
            #plt.show()
                      
        if ccf_pack[2][max_indx]>2.0:
            peak_pvalue = corsig(ccf_pack[0][max_indx], float(ccf_pack[2][max_indx]-2.0))
        else:
            peak_pvalue = 1.0
        
        #Now, if the centroid fails, re-set the peak status to 0 because we don't want to report a peak without a centroid!
        if status_centroid == 0:
            status_peak = 2
            tlag_peak = -999.0
            max_rval = -9999.0
            status_rval = 0 

    # ccf peaks-- Eric's method using a p-value test (usually not using) 
    else:
        max_indx = np.argmax(ccf_pack[0])
        if ccf_pack[2][max_indx]>2.0:
            peak_pvalue = corsig(ccf_pack[0][max_indx], float(ccf_pack[2][max_indx]-2.0))
        else:
            peak_pvalue = 1.0

        if peak_pvalue<alpha:
            max_rval = ccf_pack[0][max_indx]
            tlag_peak = ccf_pack[1][max_indx]
            status_peak = 1
            status_rval = 1
        else:
            max_rval = -9999.0
            tlag_peak = -9999.0
            status_peak = 0
            status_rval = 0
            tlag_centroid = -9999.0
            status_centroid = 0
    
        if status_peak == 1:
            rcent = thres*ccf_pack[0][max_indx]
            selcen = ccf_pack[0]>rcent # points to calculate centroid
            if np.sum(selcen)>1 and ccf_pack[0][0]<rcent and ccf_pack[0][-1]<rcent and status_peak==1:
                tlag_centroid = np.sum(ccf_pack[0][selcen]*ccf_pack[1][selcen])/np.sum(ccf_pack[0][selcen])
                status_centroid = 1
            else:
                tlag_centroid = -9999.0
                status_centroid = 0

    #print tlag_peak, status_peak, tlag_centroid, status_centroid, max_rval, status_rval
    return tlag_peak, status_peak, tlag_centroid, status_centroid, ccf_pack, max_rval, status_rval, peak_pvalue




def xcor_mc(t1, y1, dy1, t2, y2, dy2, tlagmin, tlagmax, tunit, thres=0.8, siglevel=0.95, imode=0, nsim=2048, mcmode=0, sigmode = 0.2):
    '''
    Calculate the uncertainty for the cross-correlation peak.
    
    Inputs:
        t1 -- time for light curve 1, assume increase;
        y1 -- flux for light curve 1;
        dy1 -- flux uncertainty for light curve 1;
        t2 -- time for light curve 2, assume increase;
        y2 -- flux for light curve 2;
        dy2 -- flux uncertainty for light curve 2;
        tlagmin -- minimum time lag;
        tlagmax -- maximum time lag;
        tunit -- tau step;
        thres -- lower limit of correlation coefficient when 
                 calculate centroid, default is 0.8;
        siglevel -- the required significant level of the 
                 correlation coefficient;
        imode -- cross-correlation mode: 0, twice (default); 
                 1, interpolate light curve 1;
                 2, interpolate light curve 2.
        nsim -- MC simulation trials;
        mcmode -- MC mode: 0, RSS plus FR
                  1, RSS only
                  2, FR only
        sigmode -- How to exclude non-significant peaks:
                  Will exclude all peaks with r < input value 
                  0 will exclude all peaks based on p-value significance test. 
        
    Outputs:
        tlags_peak -- tlag of peak distribution;
        tlags_centroid -- tlag of centroid distribution;
        nsuccess_peak -- success times in finding peaks;
        nfail_peak -- fail times in finding peaks;
        nsuccess_centroid -- success times in calculating centroid;
        nfail_centroid -- fail times in calculating centroid.
    '''
    numt1 = len(t1)
    numt2 = len(t2)
    tlags_peak = []
    tlags_centroid = []
    pvals = []
    nsuccess_peak = 0
    nsuccess_rvals = 0
    nfail_peak = 0
    nsuccess_centroid = 0
    nfail_centroid = 0
    nfail_rvals = 0
    max_rvals = []
    for i in xrange(nsim):
        if mcmode!=2:
            # RSS resample light curve 1
            indx1 = np.random.randint(0, numt1, numt1)
            unique1, counts1 = np.unique(indx1, return_counts=True) # sorted unique value
            t1_rss = t1[unique1]
            y1_rss = y1[unique1]
            dy1_rss = dy1[unique1]/np.sqrt(counts1)
            
            indx2 = np.random.randint(0, numt2, numt2)
            unique2, counts2 = np.unique(indx2, return_counts=True)
            t2_rss = t2[unique2]
            y2_rss = y2[unique2]
            dy2_rss = dy2[unique2]/np.sqrt(counts2)
            
        else:
            # do not apply RSS resample, rss light curve equals to original one
            t1_rss = t1 + 0.0
            y1_rss = y1 + 0.0
            dy1_rss = dy1 + 0.0
            
            t2_rss = t2 + 0.0
            y2_rss = y2 + 0.0
            dy2_rss = dy2 + 0.0
            
        
        if mcmode!=1:
            # measurement error perturbation
            t1_fr = t1_rss + 0.0
            y1_fr = np.random.normal(y1_rss, dy1_rss)
            t2_fr = t2_rss + 0.0
            y2_fr = np.random.normal(y2_rss, dy2_rss)
        else:
            # do not aplly the error perturbation
            t1_fr = t1_rss + 0.0
            y1_fr = y1_rss + 0.0
            t2_fr = t2_rss + 0.0
            y2_fr = y2_rss + 0.0
        
        # perform CCF
        pc_pack = peakcent(t1_fr, y1_fr, t2_fr, y2_fr, tlagmin, tlagmax, tunit, thres, siglevel, imode, sigmode = sigmode)
        
        # ccf peaks
        if pc_pack[1]==1:
            tau_peak = pc_pack[0]
            tlags_peak.append(tau_peak)
            pval = pc_pack[7]
            pvals.append(pval)
            nsuccess_peak += 1
        elif pc_pack[1] == 0:
            nfail_peak += 1
        
        # ccf centroids
        if pc_pack[3]==1:
            tau_centroid = pc_pack[2]
            tlags_centroid.append(tau_centroid)
            nsuccess_centroid += 1
        else:
            nfail_centroid += 1
            
        # max_rvalues
        if pc_pack[6]==1:
            max_rvals.append(pc_pack[5])
            nsuccess_rvals += 1
        else:
            nfail_rvals += 1

    
    tlags_peak = np.asarray(tlags_peak)
    tlags_centroid = np.asarray(tlags_centroid)
    print 'Failed centroids: ', nfail_centroid
    print 'Failed peaks: ', nfail_peak
    
    return tlags_peak, tlags_centroid, nsuccess_peak, nfail_peak, nsuccess_centroid, nfail_centroid, max_rvals, nfail_rvals, pvals



#### code to randomly resample a set of input light curves
#10/9/2017 sampcode 3 update includes sampmin the minum spacing between data points
#sampcode 4, dtave indicates the minimum space between data points, data points will be selected (with no interpolation) from the parent sample, skipping points until the minimum spacing dtave is achieved
##avesamp is the average length of time between the random samples
#set dir = '' and fname=[''] to have this function work on datin[nt,3] and output array rather than save to file

#new 10/9/2017 added option dtsdin need mean and standard deviation of spacing between points e.g setting
#dtsdin very small will give very regularly spaced points.
#if negative then the absolute value is the fraction relative to the mean e.g -0.2 will set the 
#standard deviation as a fifth of the mean spacing between points


import numpy as np
import os
import sys
import astropy_stark.myrandom as mr


def myresample(dir,fname,dtave,dtsdin = -0.2, sampmin=0.8,sampcode=3,datin=[]):
 
 if (dtsdin < 0):
  dtsd = np.abs(dtsdin)*dtave
 else:
  dtsd = dtsdin
 
 dirpy=os.getcwd()
 #dir   = sys.argv[1] #the directory storing the light curves e.g '../fort/fortcode/fakelc/kep_18mar'
 #fname = sys.argv[2] # a list of the files .e.g ['file1.dat','file2.dat','...'] etc
 #dtave = sys.argv[3] #e.g 0.5 will resample with mean half day cadence
 #sampmin   = 0.8
 #sampcode = 2
 
 #!!### user arguments above. Don't change sampcode or sampmin unless you know what they do (I don't and I wrote the code).###
 if (dir != ''):
  os.chdir(dir)
 
 Nfile=len(fname)
 
 for ifile in range(Nfile):
     if (fname[ifile] == ''):
      dat = datin
     else:
      dat=np.loadtxt(fname[ifile])
     t=dat[:,0]
     x=dat[:,1]
     sig=dat[:,2]
     Ndat=t.shape[0]
     dt = (t[-1] - t[0])/(Ndat-1)
     
 # below are two versions of the code (the 2nd should be more sophisticated and consider the approximate spacing between each point when making its idxsamp selection    
     if sampcode == 1:
     	nidx=(1.-sampmin)*np.random.ranom_sample(1)[0]+sampmin
     	idxsamp=np.random.rand(low=0,high=Ndat,size=nidx)
     	datsamp=np.zeros((nidx,3))
     	datsamp[:,0]=t[idxsamp]
     	datsamp[:,1]=x[idxsamp]
     	datsamp[:,2]=sig[idxsamp]
     	
     elif sampcode == 2:
         idxcount=0
         tthen=t[0]
         idxsamp=[]
         xn   = []
         sign = []
         tn   = []
         while (idxcount < Ndat) & (tthen < t[-1]):
             
 
             a = np.random.randn(1)*dt*2
             
             tnow = tthen + dtave + a
             tn.append(tnow)
             xn.append(np.interp([tnow],t,x)[0])
             sign.append(np.interp([tnow],t,sig)[0])
             #idxsamp.append(np.abs(t-tnow).argmin()) ## index of closest time to tnow
             
             tthen=tnow
             idxcount=idxcount+1
             
         #idxsamp=np.array(idxsamp)
         tn = np.array(tn)
         xn = np.array(xn)
         sign = np.array(sign)
         nn = xn.shape[0]
         datsamp=np.zeros((nn,3))
         datsamp[:,0]=tn[:,0]
         datsamp[:,1]=xn[:,0]
         datsamp[:,2]=sign[:,0]
 
 
 
     elif sampcode == 3:
         #print 'ararar'
         idxcount=0
         tthen=t[0]
         idxsamp=[]
         tlast = t[-1]
         while (idxcount < Ndat-1) & (tthen < tlast - 4*sampmin):
             
             
             #a = np.random.randn(1)*dtave
             a = np.random.normal(dtave,dtsd,1)[0]
             
             tnow = tthen +  np.abs(a)
             idxtemp = np.abs(t-tnow).argmin()
             
             #print tnow,tthen,'before mrs'
             #print tnow, tthen, idxcount, sampmin
             if ((idxtemp not in idxsamp) and ((tnow - tthen > sampmin) or (tnow > tlast - sampmin))):
              idxsamp.append(idxtemp) ## index of closest time to tnow
              idxcount=idxcount+1
              a= 1.*tthen-tnow
              tthen=tnow
             
             #print idxcount, Ndat, tthen, t[-1],'mrs'
         idxsamp=np.array(idxsamp)
         datsamp=np.zeros((idxsamp.shape[0],3))
         datsamp[:,0]=t[idxsamp]
         datsamp[:,1]=x[idxsamp]
         datsamp[:,2]=sig[idxsamp]
         
         ttemp_space = datsamp[1:,0]-datsamp[:-1,0]
         #print('min,max,and ave spacing between elements',ttemp_space.min(), ttemp_space.max(), np.mean(ttemp_space))
 
     
 
 
     elif sampcode == 4:
         idxcount=0
         tthen=t[0]
         idxsamp=[]
         while (idxcount < Ndat) & (tthen < t[-1]):
             
             
             tnow = tthen +  dtave
             
             b = t>tnow
             idxtemp = [i for i, elem in enumerate(b, 1) if elem]
             if (len(idxtemp) ==0):
              break
             
             idxtemp = idxtemp[0]
             if (idxtemp >= t.shape[0]):
              break
              
             if (idxtemp not in idxsamp):
              idxsamp.append(idxtemp) ## index of closest time to tnow
              idxcount=idxcount+1
             
             a= tnow - tthen
             tthen=t[idxtemp]
             
             #if (t[idxtemp] - t[idxtemp-1] < dtave):
              #print 'PROBLEM!!',
              #print t[idxtemp] - t[idxtemp-1], dtave#print tnow, idxcount, dtave, dt, tthen, a
              #raw_input()
         #print idxsamp
         idxsamp=np.array(idxsamp)
         datsamp=np.zeros((idxsamp.shape[0],3))
         datsamp[:,0]=t[idxsamp]
         datsamp[:,1]=x[idxsamp]
         datsamp[:,2]=sig[idxsamp]
         
         ttemp_space = datsamp[1:,0]-datsamp[:-1,0]
         ns = len(idxsamp)
         #for i in range(ns):
         # print i,datsamp[i,:]
         #print('min,max,and ave spacing between elements',ttemp_space.min(), ttemp_space.max(), np.mean(ttemp_space))
         #print('locations...',ttemp_space.argmin(),ttemp_space.argmax())
 
 
     
     np.savetxt('resamp_'+fname[ifile],datsamp)   
 
 os.chdir(dirpy) # change back to python directory
 return(datsamp)
 
 
 
 
 




#return the uncertainties from cream output parameters (default returns mdot, deginc and tr slope)

import numpy as np
import glob
import myreadlines as mr


def gcp(dir,dirsub='',paridx=[2,3,4],idbi = 2./3,conlims=[15.865,50,84.135],deginc=1,idcos = 3):
 
 
 if (dirsub == ''):
  #print 'get_cream_pars   looking for cream output in '+dir+'/output_20*'
  dirnow = sorted(glob.glob(dir+'/output_20*'))[-1]
 else:
  dirnow = dir+'/'+dirsub
 
 print 'get_cream_pars   looking for cream output in '+dirnow
 
 try:
  pload = np.loadtxt(dirnow+'/outputpars.dat')
 except:
  pload = mr.myreadlines(dirnow+'/outputpars.dat')[0]
 
 ndat = np.shape(pload)[0]
 
 itlo = np.int(ndat*idbi)
 pload = pload[itlo:,paridx]
 
 
 iddeg = np.where(np.array(paridx) == idcos)[0]
 if (np.shape(iddeg)[0] > 0):
  pload[:,iddeg[0]] = np.arccos(pload[:,iddeg[0]])*180/np.pi 
 
 oplo,opmed,ophi = np.percentile(pload,conlims,axis=0)
 
 return(oplo,opmed,ophi)
 


import numpy as np



def mymc_lc(taunow,lc,sig,nits = 100,means=0):

 
 ndat = np.shape(lc)[0]
 lcsave = np.zeros((ndat,nits))
 
 taumeansave = []
 for it in range(nits):
  lcnow = lc + np.random.randn(ndat)*sig
  lcsave[:,it] = lcnow
  if (means == 1):
   taumean = np.sum(lcnow*taunow)/np.sum(lcnow)
   taumeansave.append(taumean)
   
 lcout = np.array(np.percentile(lcsave,[15.865,50,84.315],axis = 1)).T
 if (means == 0):
  return(lcout)
 else:
  taumeansave = np.array(taumeansave)
  means_out = np.percentile(taumeansave,[15.865,50,84.315])
  return(lcout,means_out)